-- The main function is the first function called from Iguana.
-- The Data argument will contain the message to be processed.
-- Note: main() is the last function at the end of this code.

local cda = require 'cda'

require 'cda.xml'
require 'cda.null'
require 'cda.codeset'

local addElement = node.addElement
local setAttr = node.setAttr
local setInner = node.setInner
local setText = node.setText

-- Fill out empty attributes/elements in the header
-- all the values here are customer specific, you will
-- need to customise them to suit your requirements
local function FillHeader(CD)
   -- set the attributes
   CD.id.root = '2.16.840.1.113883.19.5.99999.2'
   CD.id.extension = 'TT998'
   CD.setId.root = '2.16.840.1.113883.19.7'
   CD.setId.extension = 'BB35'
   CD.versionNumber.value = 2   
   CD.languageCode.code = cda.codeset.language["English - US"]
   
   -- set the elements 
   setText(CD.title, 'Good Health Clinic Consultation Note')  
   cda.time.set{target=CD.effectiveTime, time='20070415000000+0500'}
   cda.code.set{target=CD.code, system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc["Consultative note"], lookup=cda.codeset.loinc.reverse}
   cda.code.set{target=CD.confidentialityCode, system=cda.codeset.cat.Confidentiality, 
      value=cda.codeset.confidentiality['Normal'], lookup=cda.codeset.confidentiality.reverse}  

   return CD
end

local function FillPatient(RT)
   local function FillGuardian(G)
      cda.code.add{target=G, element='code', system=cda.codeset.cat["HL7 Role Class"],
         value=cda.codeset.personalRelationshipRole.Parent, lookup=cda.codeset.personalRelationshipRole.reverse}
      cda.demographic.address.add{target=G, use=cda.codeset.address.Home, 
         street='1357 Amber Drive', city='Beaverton', state='OR', zip='97867', country='US'}
      cda.demographic.phone.add{target=G, phone='(816)276-6909', use=cda.codeset.address.Home}
      local GP = addElement(G, 'guardianPerson')
      cda.demographic.name.add{target=GP, given='Ralph', family='Jones'}
      
      return G
   end
   
   local function FillBirthPlace(B)
      local P = addElement(B, 'place')
      cda.demographic.address.add{target=P, city='Beaverton', state='OR', zip='97867', country='US'}  
      
      return B
   end
   
   local function FillLanguageCommunication(L)
      cda.code.simple.add{target=L, element='languageCode', value=cda.codeset.language['English - US']}
      cda.code.add{target=L, element='modeCode', system=cda.codeset.cat["LanguageAbilityMode"],
         value=cda.codeset.proficiencyLevel["Good"], lookup=cda.codeset.proficiencyLevel.reverse}
      cda.value.add{target=L, element='preferenceInd', datatype='BL', value='true'}
      return L
   end
   
   local function FillProviderOrganization(O)
      cda.id.add{target=O, id_type=cda.codeset.cat["National Provider Identifier"]}
      cda.demographic.name.simple.add{target=O, name='Community Health and Hospitals'}
      cda.demographic.phone.add{target=O, phone='(555)555-5000', use=cda.codeset.address.Work}
      cda.demographic.address.add{target=O, use=cda.codeset.address.Work, 
         street='1001 Village Avenue', city='Beaverton', state='OR', zip='99123', country='US'}  
      
      return O
   end
   
   local PR = addElement(RT, 'patientRole')
   cda.id.add{target=PR, value='998991', id_type='2.16.840.1.113883.4.6'} -- dummy OID change or map value
   cda.id.add{target=PR, value='111-00-2330', id_type=cda.codeset.cat.SSN}
   cda.demographic.address.add{target=PR, use=cda.codeset.address.Home, street='1357 Amber Drive', 
      city='Beaverton', state='OR', zip='97867', country='US'} 
   cda.demographic.phone.add{target=PR, phone='(816)276-6909', use=cda.codeset.address.Home}  
   local P = addElement(PR, 'patient')
   cda.demographic.name.add{target=P, given='Isabella', nickname='Isa', family='Jones', use=cda.codeset.nameUses.Legal}
   cda.code.add{target=P, element='administrativeGenderCode', system=cda.codeset.cat["HL7 AdministrativeGender"], 
      value=cda.codeset.sex.Female, lookup=cda.codeset.sex.reverse}
   cda.time.add{target=P, element='birthTime', time='19750501000000+0500'}   
   cda.code.add{target=P, element='maritalStatusCode', system=cda.codeset.cat["HL7 Marital status"],
      value=cda.codeset.marriage.Married, lookup=cda.codeset.marriage.reverse}
   cda.code.add{target=P, element='religiousAffiliationCode', system=cda.codeset.cat.ReligiousAffiliation,
      value=cda.codeset.religion.Atheism, lookup=cda.codeset.religion.reverse}
   cda.code.add{target=P, element='raceCode', system=cda.codeset.cat["HL7 Race and Ethnicity"],
      value=cda.codeset.race.White, lookup=cda.codeset.race.reverse}
   cda.code.add{target=P, element='ethnicGroupCode', system=cda.codeset.cat["HL7 Race and Ethnicity"], 
      value=cda.codeset.ethnicGroup["Not Hispanic or Latino"], 
      lookup=cda.codeset.ethnicGroup.reverse}

   local G = addElement(P, 'guardian')
   FillGuardian(G)  
   local BP = addElement(P, 'birthplace')
   FillBirthPlace(BP)
   local LC = addElement(P, 'languageCommunication')
   FillLanguageCommunication(LC)
   local PO = addElement(PR, 'providerOrganization')
   FillProviderOrganization(PO)
   
   return RT
end

local function FillAuthor(A)
   cda.time.add{target=A, element='time', time='20070415000000+0500'}
   local AA = addElement(A, 'assignedAuthor')
   cda.id.add{target=AA, value='99999999', id_type=cda.codeset.cat["National Provider Identifier"]}
   cda.code.add{target=AA, element='code', system=cda.codeset.cat["Provider Codes"],
      value=cda.codeset.providerCodes["Allopathic &amp; Osteopathic Physicians"], lookup=cda.codeset.providerCodes.reverse}  
   cda.demographic.address.add{target=AA, street='1002 Healthcare Drive', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AA, phone='(555)555-1002', use=cda.codeset.address.Work} 
   local AP = addElement(AA, 'assignedPerson')
   cda.demographic.name.add{target=AP, given='Henry', family='Seven'}
   
   return A
end

local function FillDataEnterer(D)
   local AE = addElement(D, 'assignedEntity')
   cda.id.add{target=AE, value='999999943252', id_type=cda.codeset.cat["National Provider Identifier"]}
   cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AE, phone='(555)555-1002', use=cda.codeset.address.Work}   
   local AP = addElement(AE, 'assignedPerson')
   cda.demographic.name.add{target=AP, given='Henry', family='Seven'}
   
   return D   
end

local function FillInformant1(I)
   local AE = addElement(I, 'assignedEntity')   
   cda.id.add{target=AE, value='KP00017', id_type='2.16.840.1.113883.19.5'}
   cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AE, phone='(555)555-1002', use=cda.codeset.address.Work}   
   local AP = addElement(AE, 'assignedPerson')
   cda.demographic.name.add{target=AP, given='Henry', family='Seven'}
   
   return I   
end

local function FillInformant2(I)
   local RE = addElement(I, 'relatedEntity')   
   setAttr(RE, 'classCode', 'PRS') 
   cda.code.add{target=RE, element='code', system=cda.codeset.cat["Personal Relationship Role"],
      value=cda.codeset.personalRelationshipRole.spouse, lookup=cda.codeset.personalRelationshipRole.reverse}
   local RP = addElement(RE, 'relatedPerson')
   cda.demographic.name.add{target=RP, given='Frank', family='Jones'}
   
   return I      
end

local function FillCustodian(C)
   local AC = addElement(C, 'assignedCustodian')
   local O = addElement(AC, 'representedCustodianOrganization')
   cda.id.add{target=O, value='99999999', id_type=cda.codeset.cat["National Provider Identifier"]}
   cda.demographic.name.simple.add{target=O, name='Community Health and Hospitals'}
   cda.demographic.phone.add{target=O, phone='(555)555-1002', use=cda.codeset.address.Work} 
   cda.demographic.address.add{target=O, use=cda.codeset.address.Work, street='1002 Healthcare Drive', 
      city='Portland', state='OR', zip='99123', country='US'}  
   
   return C
end

local function FillInformationRecipient(I)
   local IR = addElement(I, 'intendedRecipient')
   cda.demographic.name.add{target=addElement(IR, 'informationRecipient'), given='Henry', family='Seven'}  
   local O = addElement(IR, 'receivedOrganization')
   cda.demographic.name.simple.add{target=O, name='Community Health and Hospitals'}

   return I
end

local function FillLegalAuthenticator(L)
   cda.time.add{target=L, element='time', time='20070415000000+0500'}
   cda.code.simple.add{target=L, element='signatureCode', value=cda.codeset.signature['Signed']} 
   local addElement = node.addElement   
   local AE = addElement(L, 'assignedEntity')
   cda.id.add{target=AE, value='999999999', id_type=cda.codeset.cat["National Provider Identifier"]} 
   cda.demographic.address.add{target=AE,street='1001 Village Avenue', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AE, phone='(555)555-1002', use=cda.codeset.address.Work} 
   local AP = addElement(AE, 'assignedPerson')
   cda.demographic.name.add{target=AP, given='Henry', family='Seven'}
   
   return L
end

local function FillAuthenticator(A)
   cda.time.add{target=A, element='time', time='20070415000000+0500'}
   cda.code.simple.add{target=A, element='signatureCode', value=cda.codeset.signature['Signed']} 
   local addElement = node.addElement   
   local AE = addElement(A, 'assignedEntity')
   cda.id.add{target=AE, value='999999999', id_type=cda.codeset.cat["National Provider Identifier"]}
   cda.demographic.address.add{target=AE,street='1001 Village Avenue', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AE, phone='(555)555-1002', use=cda.codeset.address.Work} 
   local AP = addElement(AE, 'assignedPerson')
   cda.demographic.name.add{target=AP, given='Henry', family='Seven'}
   
   return A
end
   
local function FillServiceEvent(D)
   local addElement = node.addElement   
   local S = addElement(D, 'serviceEvent')
   setAttr(S, 'classCode', 'PCPR') 
   cda.code.add{target=S, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT.Colonoscopy, lookup=cda.codeset.snomedCT.reverse}
   local ET = cda.timerange.add{target=S, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20070415000000+0500'}
   cda.time.add{target=ET, element='high', time='20070416000000+0500'} 
   local P = addElement(S, 'performer')
   setAttr(P, 'typeCode', 'PRF') 
   cda.code.add{target=P, element='functionCode', system=cda.codeset.cat["HL7 Provider Role"],
      value=cda.codeset.providerRoles["Primary Care Provider"], lookup=cda.codeset.providerRoles.reverse}
   local T = addElement(P.functionCode, 'originalText')
   setText(T, 'Primary Care Provider') 
   local AE = addElement(P, 'assignedEntity')
   cda.id.add{target=AE, value='PseudoMD-1', id_type=cda.codeset.cat["National Provider Identifier"]}
   cda.code.add{target=AE, element='code', system=cda.codeset.cat["Provider Codes"], 
      value=cda.codeset.providerCodes.Gastroenterologist, lookup=cda.codeset.providerCodes.reverse}
   cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland', 
      state='OR', zip='99123', country='US'}  
   cda.demographic.phone.add{target=AE, phone='(555)555-1002', use=cda.codeset.address.Home} 
   local AP = addElement(AE, 'assignedPerson')
   cda.demographic.name.add{target=AP, prefix='Dr.', given='Henry', family='Seven'}
   local O = addElement(AE, 'representedOrganization')   
   cda.id.add{target=O, id_type='2.16.840.1.113883.19.5.9999.1393'}
   cda.demographic.name.simple.add{target=O, name='Community Health and Hospitals'}
   cda.demographic.phone.add{target=O, phone='(555)555-5000', use=cda.codeset.address.Home}
   cda.demographic.address.add{target=O, street='1001 Village Avenue', city='Beaverton', 
      state='OR', zip='99123', country='US'}  
      
   return S
end

local function FillAdvancedDirectives(T)
   local function FillParticipant1(Root)
      local P = addElement(Root, 'participant')
      cda.id.template.add{target=P, 
         id_type=cda.codeset.templates["Advanced Directives Verification"]}
      cda.time.add{target=P, element='time', time='20110213000000+0500'}
      local PR = addElement(P, 'participantRole')
      cda.id.add{target=PR, id_type='20cf14fb-b65c-4c8c-a54d-b0cca834c18c'}
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.add{target=PE, prefix='Dr.', given='Robert', 
         family='Dolin', element='name'}
      setAttr(P, 'typeCode', 'VRF')
      
      return P
   end
   
   local function FillParticipant2(Root)
      local P = addElement(Root, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.demographic.address.add{target=PR, street='21 North Ave.', 
         city='Burlington', state='MA', zip='02368', country='US'}
      cda.demographic.phone.add{target=PR, phone='(555)555-1003'}
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.add{target=PE, prefix='Dr.', given='Robert', 
         family='Dolin', element='name'}
      setAttr(P, 'typeCode', 'CST')
      setAttr(PR, 'classCode', 'AGNT')
      
      return P
   end
   
   local function FillReference(Root)
      local R = setAttr(addElement(Root, 'reference'), 'typeCode', 'REFR')
      cda.value.add{target=R, element='seperatableInd', datatype='BL', value='false'}
      local ED = addElement(R, 'externalDocument')
      cda.id.add{target=ED, id_type='b50b7910-7ffb-4f4c-bbe4-177ed68cbbf3'}
      local TXT = setAttr(addElement(ED, 'text'), 'mediaType', 'application/pdf')
      cda.value.add{target=TXT, element='reference', datatype='ST', 
         value='AdvanceDirective.b50b7910-7ffb-4f4c-bbe4-177ed68cbbf3.pdf'}
      
      return R
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Advanced Directives With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Advanced Directives"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'ADVANCED DIRECTIVES')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Directive</th>
                  <th>Description</th>
                  <th>Verification</th>
                  <th>Supporting Document(s)</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>Resuscitation status</td>
                  <td>
                      <content ID="AD1">Do not resuscitate</content>
                  </td>
                  <td>Dr. Robert Dolin, Feb 13, 2011</td>
                  <td>
                      <linkHtml
                       href="AdvanceDirective.b50b7910-7ffb-4f4c-bbe4-177ed68cbbf3.pdf"
                      >Advance directive</linkHtml>
                  </td>
              </tr>
          </tbody>
      </table>
      ]])
   
   local OB = addElement(addElement(S, 'entry'), 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Advanced Directives Observation"]}
   cda.id.add{target=OB, id_type='9b54c3c9-1673-49c7-aef9-b037ed72ed27'}
   cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
      value=cda.codeset.advancedDirectivesTypes.Resuscitation, 
      lookup=cda.codeset.advancedDirectivesTypes.reverse}
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
   local ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20070415000000+0500'} 
   cda.null.set(cda.time.add{target=ET, element='high'}, cda.null.flavor.NotApplicable)
   local V = cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Not for resuscitation"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.originalText.add{target=V, reference='#AD1'}
   
   FillParticipant1(OB)
   FillParticipant2(OB)
   FillReference(OB)
   
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
   
   return T
end

local function FillAllergies(T)
   local function FillAllergyProblem1(S)
      local function FillStatus(Root)
         local ER = addElement(Root, 'entryRelationship')
         local OB = addElement(ER, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Status Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
            value=cda.codeset.loinc.Status, lookup=cda.codeset.loinc.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Inactive"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillSeverity(OB)
         local PR = addElement(OB, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity4"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Mild"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Suceptible, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN') 
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillReaction(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Reaction Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a64'}
         cda.null.set(cda.code.add{target=OB, element='code'}, cda.null.flavor.Unknown)
         setInner(addElement(OB, 'text'), '<reference value="#reaction1"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20070501000000+0500'}    
         cda.time.add{target=ET, element='high', time='20090227000000+0500'}           
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Nausea"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(PR, 'typeCode', 'MFST'), 'inversionInd', 'true')
         
         FillSeverity(OB)
         
         return OB
      end
      
      local function FillSeverityObservation(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity2"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Moderate to severe"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN') 
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillObservation(Root)
         local ER = addElement(Root, 'entryRelationship')
         local OB = addElement(ER, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a66'}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20070501000000+0500'}
         local V = cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Propensity to adverse reactions to drug"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.originalText.add{target=V, reference='#reaction1'}
         local PV = addElement(OB, 'participant')
         local PR = addElement(PV, 'participantRole')
         local PE = addElement(PR, 'playingEntity')
         local C = cda.code.add{target=PE, element='code', system=cda.codeset.cat.RxNorm,
            value=cda.codeset.rxnorm["ALLERGENIC EXTRACT, PENICILLIN"], lookup=cda.codeset.rxnorm.reverse}
         cda.code.originalText.add{target=C, reference='#reaction1'}  
         setAttr(PV, 'typeCode', 'CSM')
         setAttr(PR, 'classCode', 'MANU')
         setAttr(PE, 'classCode', 'MMAT')
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         FillSeverityObservation(OB)    
         
         return OB
      end
      
      local E = addElement(S, 'entry')
      local ACT = addElement(E, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Allergy Problem Act"]}
      cda.id.add{target=ACT, id_type='36e3e930-7b14-11db-9fe1-0800200c9a66'}
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.LOINC,
         value=cda.codeset.loinc["Allergies, Adverse Reactions, Alerts"], 
         lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Active']}
      cda.time.add{target=ACT, element='effectiveTime', time='20070501000000+0500'}
      
      FillObservation(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')
      
      return S
   end

   local function FillAllergyProblem2(S)
      local function FillStatus(Root)
         local ER = addElement(Root, 'entryRelationship')
         local OB = addElement(ER, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Status Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
            value=cda.codeset.loinc.Status, lookup=cda.codeset.loinc.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Active"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillSeverity(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity5"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Mild"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Suceptible, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN') 
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillReaction(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Reaction Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a64'}
         cda.null.set(cda.code.add{target=OB, element='code'}, cda.null.flavor.Unknown)
         setInner(addElement(OB, 'text'), '<reference value="#reaction2"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20060501000000+0500'}
         cda.time.add{target=ET, element='high', time='20090227000000+0500'}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Wheezing"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')   
         setAttr(setAttr(PR, 'typeCode', 'MFST'), 'inversionInd', 'true')
         
         FillSeverity(OB)
         
         return OB
      end
      
      local function FillSeverityObservation(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity2"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Moderate"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')  
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillObservation(Root)
         local ER = addElement(Root, 'entryRelationship')
         local OB = addElement(ER, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a66'}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20060501000000+0500'}
         local V = cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Propensity to adverse reactions to drug"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.originalText.add{target=V, reference='#reaction2'}
         local PV = addElement(OB, 'participant')
         local PR = addElement(PV, 'participantRole')
         local PE = addElement(PR, 'playingEntity')
         local C = cda.code.add{target=PE, element='code', system=cda.codeset.cat.RxNorm,
            value=cda.codeset.rxnorm["Codeine"], lookup=cda.codeset.rxnorm.reverse}
         cda.code.originalText.add{target=C, reference='#reaction2'}
         
         FillStatus(OB)
         FillReaction(OB)
         FillSeverityObservation(OB)    
         
         setAttr(PV, 'typeCode', 'CSM')
         setAttr(PR, 'classCode', 'MANU')
         setAttr(PE, 'classCode', 'MMAT')
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local E = addElement(S, 'entry')
      local ACT = addElement(E, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Allergy Problem Act"]}
      cda.id.add{target=ACT, id_type='36e3e930-7b14-11db-9fe1-0800200c9a66'}
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.LOINC,
         value=cda.codeset.loinc["Allergies, Adverse Reactions, Alerts"], 
         lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Active']}
      cda.time.add{target=ACT, element='effectiveTime', time='20060501000000+0500'}
      
      FillObservation(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')
      
      return S
   end

   local function FillAllergyProblem3(S)
      local function FillStatus(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Status Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
            value=cda.codeset.loinc.Status, lookup=cda.codeset.loinc.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Active"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillSeverity(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity6"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Mild to moderate"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Suceptible, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')   
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local function FillReaction(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Reaction Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a64'}
         cda.null.set(cda.code.add{target=OB, element='code'}, cda.null.flavor.Unknown)
         setInner(addElement(OB, 'text'), '<reference value="#reaction3"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20080501000000+0500'}
         cda.time.add{target=ET, element='high', time='20090227000000+0500'}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Hives"], lookup=cda.codeset.snomedCT.reverse}
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')   
         setAttr(setAttr(PR, 'typeCode', 'MFST'), 'inversionInd', 'true')
         
         FillSeverity(OB)
         
         return OB
      end
      
      local function FillSeverityObservation(Root)
         local PR = addElement(Root, 'entryRelationship')
         local OB = addElement(PR, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Severity Observation"]}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act["Severity Observation"], lookup=cda.codeset.act.reverse}
         setInner(addElement(OB, 'text'), '<reference value="#severity3"/>')
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Mild to moderate"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.add{target=OB, element='interpretationCode', 
            system=cda.codeset.cat["HL7 Observation Interpretation"],
            value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}   
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')     
         setAttr(setAttr(PR, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end   
      
      local function FillObservation(Root)
         local ER = addElement(Root, 'entryRelationship')
         local OB = addElement(ER, 'observation')
         cda.id.template.add{target=OB, 
            id_type=cda.codeset.templates["Allergy Observation"]}
         cda.id.add{target=OB, id_type='4adc1020-7b14-11db-9fe1-0800200c9a66'}
         cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
            value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
         cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
         local ET = cda.timerange.add{target=OB, element='effectiveTime'}
         cda.time.add{target=ET, element='low', time='20080501000000+0500'}
         local V = cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
            value=cda.codeset.snomedCT["Drug intolerance"], lookup=cda.codeset.snomedCT.reverse}
         cda.code.originalText.add{target=V, reference='#reaction3'}
         local PV = addElement(OB, 'participant')
         local PR = addElement(PV, 'participantRole')
         local PE = addElement(PR, 'playingEntity')
         local C = cda.code.add{target=PE, element='code', system=cda.codeset.cat.RxNorm,
            value=cda.codeset.rxnorm["Aspirin"], lookup=cda.codeset.rxnorm.reverse}
         cda.code.originalText.add{target=C, reference='#reaction3'} 
         
         FillStatus(OB)
         FillReaction(OB)
         FillSeverityObservation(OB) 
         
         setAttr(PV, 'typeCode', 'CSM')
         setAttr(PR, 'classCode', 'MANU')
         setAttr(PE, 'classCode', 'MMAT')
         setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
         setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
         
         return OB
      end
      
      local E = addElement(S, 'entry')
      local ACT = addElement(E, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Allergy Problem Act"]}
      cda.id.add{target=ACT, id_type='36e3e930-7b14-11db-9fe1-0800200c9a66'}
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.LOINC,
         value=cda.codeset.loinc["Allergies, Adverse Reactions, Alerts"], 
         lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Active']}
      cda.time.add{target=ACT, element='effectiveTime', time='20080501000000+0500'}
      
      FillObservation(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')
      
      return S
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Allergies, Adverse Reactions, Alerts With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Allergies, Adverse Reactions, Alerts"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'ALLERGIES, ADVERSE REACTIONS, ALERTS')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Substance</th>
                  <th>Overall Severity</th>
                  <th>Reaction</th>
                  <th>Reaction Severity</th>
                  <th>Status</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>ALLERGENIC EXTRACT, PENICILLIN</td>
                  <td>
                      <content ID="severity4">Moderate to Severe</content>
                  </td>
                  <td>
                      <content ID="reaction1">Nausea</content>
                  </td>
                  <td>
                      <content ID="severity1">Mild</content>
                  </td>
                  <td>Inactive</td>
              </tr>
              <tr>
                  <td>Codeine</td>
                  <td>
                      <content ID="severity5">Mild</content>
                  </td>
                  <td>
                      <content ID="reaction2">Wheezing</content>
                  </td>
                  <td>
                      <content ID="severity2">Moderate</content>
                  </td>
                  <td>Active</td>
              </tr>
              <tr>
                  <td>Aspirin</td>
                  <td>
                      <content ID="severity6">Mild</content>
                  </td>
                  <td>
                      <content ID="reaction3">Hives</content>
                  </td>
                  <td>
                      <content ID="severity3">Mild to moderate</content>
                  </td>
                  <td>Active</td>
              </tr>
          </tbody>
      </table>
      ]])  
   
   FillAllergyProblem1(S)
   FillAllergyProblem2(S)
   FillAllergyProblem3(S)

   return T
end

local function FillAssessment(T)
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Assessment"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Assessment"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'ASESSMENT')
   setInner(addElement(S, 'text'), [[
      <list listType="ordered">
          <item>Recurrent GI bleed of unknown etiology; hypotension perhaps
          secondary to this but as likely secondary to polypharmacy.</item>
          <item>Acute on chronic anemia secondary to #1.</item>
          <item>Azotemia, acute renal failure with volume loss secondary to
          #1.</item>
          <item>Hyperkalemia secondary to #3 and on ACE and K+ supplement.</item>
          <item>Other chronic diagnoses as noted above, currently stable.</item>
      </list>
      ]])
   
   return T
end

local function FillEncounters(T)
   local function FillParticipant(Root)
      local P = addElement(Root, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Service Delivery Location"]}
      cda.code.add{target=PR, element='code', system=cda.codeset.cat["Healthcare Service Location"],
         value=cda.codeset.healthcareServiceLocation["Urgent Care Center"], 
         lookup=cda.codeset.healthcareServiceLocation.reverse}
      cda.demographic.address.add{target=PR, street='17 Daws Rd.', 
         city='Blue Bell', state='MA', zip='02368', country='US'}
      cda.null.set(cda.demographic.phone.add{target=PR}, cda.null.flavor.Unknown)
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.simple.add{target=PE, name='Community Urgent Care Center'}
      setAttr(PE, 'classCode', 'PLC')
      setAttr(P, 'typeCode', 'LOC')
      setAttr(PR, 'classCode', 'SDLOC')
      
      return P
   end
   
   local function FillReason(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Indication"]}
      cda.id.add{target=OB, value='45665', id_type='db734647-fc99-424c-a864-7e3cda82e703'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Finding"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20070103000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Pneumonia"], lookup=cda.codeset.snomedCT.reverse}  
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')     
      setAttr(ER, 'typeCode', 'RSON')
      
      return ER
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Encounters With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["History of encounters"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'ENCOUNTERS')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Encounter</th>
                  <th>Performer</th>
                  <th>Location</th>
                  <th>Date</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>
                      <content ID="Encounter1"/> Checkup Examination </td>
                  <td>Performer Name</td>
                  <td>Community Urgent Care Center</td>
                  <td>20090227130000+0500</td>
              </tr>
          </tbody>
      </table>
      ]])   
   
   local E = addElement(S, 'entry')
   local EN = addElement(E, 'encounter')
   cda.id.template.add{target=EN, 
      id_type=cda.codeset.templates["Encounter Activities"]} 
   cda.id.add{target=EN, id_type='2a620155-9d11-439e-92b3-5d9815ff4de8'}
   local C = cda.code.add{target=EN, element='code', system=cda.codeset.cat.CPT, 
      value=cda.codeset.cpt["Office outpatient visit 15 minutes"], lookup=cda.codeset.cpt.reverse}
   setInner(addElement(C, 'originalText'), 'Checkup Examination <reference value="#Encounter1"/>')
   cda.code.add{target=C, element='translation', system=cda.codeset.cat["HL7 ActCode"], 
      value=cda.codeset.act["Ambulatory"], lookup=cda.codeset.act.reverse}
   setAttr(C, 'codeSystemVersion', '4')
   cda.time.add{target=EN, element='effectiveTime', time='20090227130000+0500'}
   local PERF = addElement(EN, 'performer')
   local AE = addElement(PERF, 'assignedEntity')
   cda.id.add{target=AE, id_type='PseudoMD-3'}
   cda.code.add{target=AE, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["General Physician"], lookup=cda.codeset.snomedCT.reverse}
   
   FillParticipant(EN)
   FillReason(EN)
   
   setAttr(setAttr(EN, 'classCode', 'ENC'), 'moodCode', 'EVN')
   setAttr(E, 'typeCode', 'DRIV')
   
   return T   
end

local function FillFamilyHistory(T)
   local function FillSubject(Root)
      local SUB = addElement(Root, 'subject')
      local RS = addElement(SUB, 'relatedSubject') 
      local C = cda.code.add{target=RS, element='code', system=cda.codeset.cat["Personal Relationship Role"], 
         value=cda.codeset.personalRelationshipRole.Father, lookup=cda.codeset.personalRelationshipRole.reverse}
      cda.code.add{target=C, element='translation', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Biological father"], lookup=cda.codeset.snomedCT.reverse}
      local RSSUB = addElement(RS, 'subject')
      cda.code.add{target=RSSUB, element='administrativeGenderCode', system=cda.codeset.cat["HL7 AdministrativeGender"], 
         value=cda.codeset.sex.Male, lookup=cda.codeset.sex.reverse}
      cda.time.add{target=RSSUB, element='birthTime', time='19100101000000+0500'}
      setAttr(RS, 'classCode', 'PRS')
      
      return SUB
   end
   
   local function FillDeathObservation(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Family History Death Observation"]} 
      cda.id.add{target=OB, id_type='6898fae0-5c8a-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Dead"], lookup=cda.codeset.snomedCT.reverse}  
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')     
      setAttr(ER, 'typeCode', 'CAUS')
      
      return ER
   end
   
   local function FillAgeObservation1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Age Observation"]} 
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Age at onset"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.value.add{target=OB, element='value', datatype='PQ', value='57', unit='a'}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')     
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true')
      
      return ER
   end

   local function FillObservation1(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Family History Observation"]} 
      cda.id.add{target=OB, id_type='d42ebf70-5c89-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Condition"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='19670101000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Myocardial infarction"], lookup=cda.codeset.snomedCT.reverse}
      
      FillDeathObservation(OB)
      FillAgeObservation1(OB)
      
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end
 
   local function FillAgeObservation2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Age Observation"]} 
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Age at onset"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.value.add{target=OB, element='value', datatype='PQ', value='40', unit='a'}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')     
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true') 
      
      return ER
   end
   
   local function FillObservation2(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Family History Observation"]} 
      cda.id.add{target=OB, id_type='d42ebf70-5c89-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Condition"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='19500101000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Diabetes mellitus type 2"], lookup=cda.codeset.snomedCT.reverse}  
      
      FillAgeObservation2(OB)
 
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Family History"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["History of encounters"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'FAMILY HISTORY')
   setInner(addElement(S, 'text'), [[
      <paragraph>Father (deceased)</paragraph>
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Diagnosis</th>
                  <th>Age At Onset</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>Myocardial Infarction (cause of death)</td>
                  <td>57</td>
              </tr>
              <tr>
                  <td>Diabetes</td>
                  <td>40</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   local E = addElement(S, 'entry')
   local ORG = addElement(E, 'organizer')
   cda.id.template.add{target=ORG, 
      id_type=cda.codeset.templates["Family History Organizer"]} 
   cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
   
   FillSubject(ORG)
   FillObservation1(ORG)
   FillObservation2(ORG)
   
   setAttr(setAttr(ORG, 'classCode', 'CLUSTER'), 'moodCode', 'EVN')
   setAttr(E, 'typeCode', 'DRIV')   
   
   return T   
end

local function FillFunctionalStatus(T)
   local function FillFunctionalAssessment1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Supporting Observation"]} 
      cda.id.add{target=OB, id_type='f4dce790-8328-11db-9fe1-0800200c9a33'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Repetition of Three Words"], lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Three"], lookup=cda.codeset.loinc.reverse}   
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')
      
      return ER
   end
   
   local function FillFunctionalAssessment2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Supporting Observation"]} 
      cda.id.add{target=OB, id_type='f4dce790-8328-11db-9fe1-0800200c9a22'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Temporal orientation - current year"], lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Missed by 2-5 years"], lookup=cda.codeset.loinc.reverse}   
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')
      
      return ER
   end
   
   local function FillFunctionalCaregiver(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Caregiver Characteristics"]} 
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0c'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
         value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Caregiver difficulty providing physical care"], lookup=cda.codeset.snomedCT.reverse}
      local P = addElement(OB, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.code.add{target=PR, element='code', system=cda.codeset.cat["HL7 Role Class"],
         value=cda.codeset.personalRelationshipRole.Mother, lookup=cda.codeset.personalRelationshipRole.reverse}
      setAttr(PR, 'classCode', 'CAREGIVER')
      setAttr(P, 'typeCode', 'IND')
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'REFR')
      
      return ER
   end
   
   local function FillFunctional1Relationship(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Observation"]} 
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0b'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Brief interview for mental status"], lookup=cda.codeset.loinc.reverse}
      setText(addElement(OB, 'derivationExpr'), 'Text description of the calculation')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20120214000000+0500'}
      cda.value.add{target=OB, element='value', datatype='INT', value='7'}
      
      FillFunctionalAssessment1(OB)
      FillFunctionalAssessment2(OB)
      FillFunctionalCaregiver(OB)
      
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')
      
      return ER
   end
   
   local function FillFunctionalObservation(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Result Observation"]} 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Functional Status Result Observation"]}
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0a'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.Ambulation, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#FUNC1"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20090311123000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Independently able"], lookup=cda.codeset.snomedCT.reverse} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Better, lookup=cda.codeset.interpretation.reverse} 
      
      FillFunctional1Relationship(OB)
      
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')       
      
      return OB
   end
   
   local function FillFunctional1(S)
      local E = addElement(S, 'entry')
      local ORG = addElement(E, 'organizer')
      cda.id.template.add{target=ORG, 
         id_type=cda.codeset.templates["Result Organizer"]} 
      cda.id.template.add{target=ORG, 
         id_type=cda.codeset.templates["Function Status Result Organizer"]} 
      cda.id.add{target=ORG, id_type='9295dba4-df05-46bb-b94e-f2c4e4b156f8'}
      cda.code.add{target=ORG, element='code', system=cda.codeset.cat.ICF, 
         value=cda.codeset.icf["Self-Care"], lookup=cda.codeset.icf.reverse}
      cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
      
      FillFunctionalObservation(ORG)
 
      setAttr(setAttr(ORG, 'classCode', 'CLUSTER'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end

   local function FillFunctional2(S)
      local E = addElement(S, 'entry')
      local OB = addElement(E, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Observation"]}
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Functional Status Problem Observation"]} 
      cda.id.add{target=OB, id_type='08edb7c0-2111-43f2-a784-9a5fdfaa67f0'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Finding of functional performance and activity"], lookup=cda.codeset.snomedCT.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#FUNC2" />')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20070212000000+0500'}
      cda.time.add{target=ET, element='high', time='20070410000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT.Dyspnea, lookup=cda.codeset.snomedCT.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(E, 'typeCode', 'DRIV')
      
      return E 
   end
   
   local function FillCognitive1(S)
      local E = addElement(S, 'entry')
      local OB = addElement(E, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Observation"]}
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Cognitive Status Problem Observation"]} 
      cda.id.add{target=OB, id_type='08edb7c0-2111-43f2-a784-9a5fdfaa67f0'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Cognitive function finding"], lookup=cda.codeset.snomedCT.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#COG2" />')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20070412000000+0500'}
      cda.time.add{target=ET, element='high', time='20070604000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Memory impairment"], lookup=cda.codeset.snomedCT.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
   
   local function FillCognitiveAssessment1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Supporting Observation"]} 
      cda.id.add{target=OB, id_type='f4dce790-8328-11db-9fe1-0800200c9a33'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Repetition of Three Words"], lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Three"], lookup=cda.codeset.loinc.reverse}   
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')
      
      return ER
   end

   local function FillCognitiveAssessment2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Supporting Observation"]} 
      cda.id.add{target=OB, id_type='f4dce790-8328-11db-9fe1-0800200c9a22'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Temporal orientation - current year"], lookup=cda.codeset.loinc.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Missed by 2-5 years"], lookup=cda.codeset.loinc.reverse}   
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')
      
      return ER
   end
   
   local function FillCognitiveCaregiver(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Caregiver Characteristics"]} 
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0c'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"], 
         value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Caregiver difficulty providing physical care"], lookup=cda.codeset.snomedCT.reverse}
      local P = addElement(OB, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.code.add{target=PR, element='code', system=cda.codeset.cat["HL7 Role Class"],
         value=cda.codeset.personalRelationshipRole.Mother, lookup=cda.codeset.personalRelationshipRole.reverse}
      setAttr(PR, 'classCode', 'CAREGIVER')
      setAttr(P, 'typeCode', 'IND')
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'REFR')
      
      return ER
   end
   
   local function FillCognitive2Relationship1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Assessment Scale Observation"]} 
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0b1'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Glasgow Coma score"], lookup=cda.codeset.snomedCT.reverse}
      setInner(addElement(OB, 'derivationExpr'), 'Text description of the calculation')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20120214000000+0500'}
      cda.value.add{target=OB, element='value', datatype='INT', value='7'}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
      setAttr(ER, 'typeCode', 'COMP')

      FillCognitiveAssessment1(OB)
      FillCognitiveAssessment2(OB)
      FillCognitiveCaregiver(OB)
      
      return ER
   end
   
   local function FillCognitiveObservation(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Result Observation"]} 
      cda.id.template.add{target=OB, id_type=cda.codeset.templates["Cognitive Status Result Observation"]}
      cda.id.add{target=OB, id_type='c6b5a04b-2bf4-49d1-8336-636a3813df0a'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.snomedCT["Cognitive function finding"], lookup=cda.codeset.snomedCT.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#COG1" />')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}  
      cda.time.add{target=OB, element='effectiveTime', time='20090311123000+0500'}   
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Aggressive behavior"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.add{target=OB, element='interpretationCode', 
         system=cda.codeset.cat["HL7 Observation Interpretation"],
         value=cda.codeset.interpretation["High alert"], lookup=cda.codeset.interpretation.reverse}  
      
      FillCognitive2Relationship1(OB)
      
      local OBR = addElement(addElement(OB, 'referenceRange'), 'observationRange')
      setText(addElement(OBR, 'text'), 'Non Aggressive to Aggressive') 
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')   
      
      return OB
   end
   
   local function FillCognitive2(S)
      local E = addElement(S, 'entry')
      local ORG = addElement(E, 'organizer')
      cda.id.template.add{target=ORG, 
         id_type=cda.codeset.templates["Result Organizer"]} 
      cda.id.template.add{target=ORG, 
         id_type=cda.codeset.templates["Cognitive Status Result Organizer"]} 
      cda.id.add{target=ORG, id_type='9295dba4-df05-46bb-b94e-f2c4e4b156f8'}
      cda.code.add{target=ORG, element='code', system=cda.codeset.cat.ICF, 
         value=cda.codeset.icf["Communication"], lookup=cda.codeset.icf.reverse}
      cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
      
      FillCognitiveObservation(ORG)
     
      setAttr(setAttr(ORG, 'classCode', 'CLUSTER'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')  
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Functional Status"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Functional status assessment"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'Functional Status')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Functional or Cognitive Finding</th>
                  <th>Observation</th>
                  <th>Observation Date</th>
                  <th>Condition Status</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td><content ID="FUNC1">Ambulation(Dependent to
                  Independent)</content></td>
                  <td>Independently able</td>
                  <td>March 11, 2009</td>
                  <td>Active</td>
              </tr>
              <tr>
                  <td><content ID="FUNC2">Finding of Functional Performance and
                  Activity</content></td>
                  <td>Dyspnea</td>
                  <td>February 2007</td>
                  <td>Active</td>
              </tr>
              <tr>
                  <td>Cognitive Function Finding</td>
                  <td><content ID="COG2">Memory impairment</content></td>
                  <td>April 2007</td>
                  <td>Active</td>
              </tr>
              <tr>
                  <td><content ID="COG1">Cognitive Function Finding(Non-Aggressive
                  to Aggressive)</content></td>
                  <td>Aggressive behavior</td>
                  <td>March 11, 2009</td>
                  <td>Active</td>
              </tr>
          </tbody>
      </table>
      ]])   
   
   FillFunctional1(S)
   FillFunctional2(S)
   FillCognitive1(S)
   FillCognitive2(S)
   
   return T   
end

local function FillImmunizations(T)
   local function FillConsumable1(Root)
      local MAN = addElement(addElement(Root, 'consumable'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Immunization Medication Information"]} 
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Influenza virus vaccine"], lookup=cda.codeset.cvx.reverse}
      cda.code.originalText.add{target=MC, reference='#immi1'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Influenza, seasonal, injectable"], lookup=cda.codeset.cvx.reverse}   
      setText(addElement(MAT, 'lotNumberText'), '1')   
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Health LS - Immuno Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981824', id_type='2.16.840.1.113883.19.5.9999.456'}
      cda.demographic.address.add{target=AE, street='1021 Health Drive', city='Ann Arbor', 
         state='MI', zip='99099', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, given='Amanda', family='Assigned'}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1394'}
      cda.demographic.name.simple.add{target=RO, name='Good Health Clinic'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      
      return MAN
   end
   
   local function FillImmunization1(S)
      local E = addElement(S, 'entry')
      local SA = addElement(E, 'substanceAdministration')
      cda.id.template.add{target=SA, 
         id_type=cda.codeset.templates["Immunization Activity"]}
      cda.id.add{target=SA, id_type='e6f1ba43-c0ed-4b9b-9f12-f435d8ad8f92'}
      setInner(addElement(SA, 'text'), '<reference value="#immun1" />')
      cda.code.simple.add{target=SA, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=SA, element='effectiveTime', time='19991111000000+0500'}   
      cda.code.add{target=SA, element='routeCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Intramuscular injection"], lookup=cda.codeset.nci.reverse}
      cda.value.add{target=SA, element='doseQuantity', datatype='PQ', value='50', unit='mcg'}
      
      FillConsumable1(SA)
      
      local ER = addElement(SA, 'entryRelationship')
      local ACT = addElement(ER, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Instructions"]}   
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Immunization education"], lookup=cda.codeset.snomedCT.reverse} 
      setInner(addElement(ACT, 'text'), [[<reference value="#immunSect"/> Possible flu-like 
         symptoms for three days.]])  
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'INT')       
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'false')   
      setAttr(setAttr(setAttr(SA, 'classCode', 'SBADM'), 'moodCode', 'EVN'), 'negationInd', 'false')       
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
   
   local function FillConsumable2(Root)
      local MAN = addElement(addElement(Root, 'consumable'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Immunization Medication Information"]} 
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Influenza virus vaccine"], lookup=cda.codeset.cvx.reverse}
      cda.code.originalText.add{target=MC, reference='#immi2'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Influenza, seasonal, injectable"], lookup=cda.codeset.cvx.reverse}   
      setText(addElement(MAT, 'lotNumberText'), '1')   
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Health LS - Immuno Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981824', id_type='2.16.840.1.113883.19.5.9999.456'}      
      cda.demographic.address.add{target=AE, street='1021 Health Drive', city='Ann Arbor', 
         state='MI', zip='99099', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, given='Amanda', family='Assigned'}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1394'}
      cda.demographic.name.simple.add{target=RO, name='Good Health Clinic'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown) 
      
      return MAN
   end
   
   local function FillImmunization2(S)
      local E = addElement(S, 'entry')
      local SA = addElement(E, 'substanceAdministration')
      cda.id.template.add{target=SA, 
         id_type=cda.codeset.templates["Immunization Activity"]}
      cda.id.add{target=SA, id_type='e6f1ba43-c0ed-4b9b-9f12-f435d8ad8f92'}
      setInner(addElement(SA, 'text'), '<reference value="#immun2" />')
      cda.code.simple.add{target=SA, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=SA, element='effectiveTime', time='19981215000000+0500'}   
      cda.code.add{target=SA, element='routeCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Intramuscular injection"], lookup=cda.codeset.nci.reverse}
      cda.value.add{target=SA, element='doseQuantity', datatype='PQ', value='50', unit='mcg'}
      
      FillConsumable2(SA)
      
      local ER = addElement(SA, 'entryRelationship')
      local ACT = addElement(ER, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Instructions"]}   
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Immunization education"], lookup=cda.codeset.snomedCT.reverse} 
      setInner(addElement(ACT, 'text'), [[<reference value="#immunSect"/> Possible flu-like 
         symptoms for three days.]])  
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'INT')       
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'false')   
      setAttr(setAttr(setAttr(SA, 'classCode', 'SBADM'), 'moodCode', 'EVN'), 'negationInd', 'true')       
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end

   local function FillConsumable3(Root)
      local MAN = addElement(addElement(Root, 'consumable'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Immunization Medication Information"]} 
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Pneumococcal polysaccharide vaccine"], lookup=cda.codeset.cvx.reverse}
      cda.code.originalText.add{target=MC, reference='#immi3'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Pneumococcal NOS"], lookup=cda.codeset.cvx.reverse}   
      setText(addElement(MAT, 'lotNumberText'), '1')   
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Health LS - Immuno Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981824', id_type='2.16.840.1.113883.19.5.9999.456'}      
      cda.demographic.address.add{target=AE, street='1021 Health Drive', city='Ann Arbor', 
         state='MI', zip='99099', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, given='Amanda', family='Assigned'}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1394'}
      cda.demographic.name.simple.add{target=RO, name='Good Health Clinic'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown) 
      
      return MAN
   end
   
   local function FillImmunization3(S)
      local E = addElement(S, 'entry')
      local SA = addElement(E, 'substanceAdministration')
      cda.id.template.add{target=SA, 
         id_type=cda.codeset.templates["Immunization Activity"]}
      cda.id.add{target=SA, id_type='e6f1ba43-c0ed-4b9b-9f12-f435d8ad8f92'}
      setInner(addElement(SA, 'text'), '<reference value="#immun3" />')
      cda.code.simple.add{target=SA, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=SA, element='effectiveTime', time='19981215000000+0500'}   
      cda.code.add{target=SA, element='routeCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Intramuscular injection"], lookup=cda.codeset.nci.reverse}
      cda.value.add{target=SA, element='doseQuantity', datatype='PQ', value='50', unit='mcg'}
      FillConsumable3(SA)
      
      setAttr(setAttr(setAttr(SA, 'classCode', 'SBADM'), 'moodCode', 'EVN'), 'negationInd', 'false')       
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
 
   local function FillConsumable4(Root)
      local MAN = addElement(addElement(Root, 'consumable'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Immunization Medication Information"]} 
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Tetanus and diphtheria toxoids - preservative free vaccine"], lookup=cda.codeset.cvx.reverse}
      cda.code.originalText.add{target=MC, reference='#immi4'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.CVX, 
         value=cda.codeset.cvx["Tetanus and diphtheria toxoids - preservative free"], lookup=cda.codeset.cvx.reverse}   
      setInner(addElement(MAT, 'lotNumberText'), '1')   
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Health LS - Immuno Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981824', id_type='2.16.840.1.113883.19.5.9999.456'}      
      cda.demographic.address.add{target=AE, street='1021 Health Drive', city='Ann Arbor', 
         state='MI', zip='99099', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, given='Amanda', family='Assigned'}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1394'}
      cda.demographic.name.simple.add{target=RO, name='Good Health Clinic'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      
      return MAN
   end
   
   local function FillImmunization4(S)
      local E = addElement(S, 'entry')
      local SA = addElement(E, 'substanceAdministration')
      cda.id.template.add{target=SA, 
         id_type=cda.codeset.templates["Immunization Activity"]}
      cda.id.add{target=SA, id_type='e6f1ba43-c0ed-4b9b-9f12-f435d8ad8f92'}
      setInner(addElement(SA, 'text'), '<reference value="#immun4" />')
      cda.code.simple.add{target=SA, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=SA, element='effectiveTime', time='19981215000000+0500'}   
      cda.code.add{target=SA, element='routeCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Intramuscular injection"], lookup=cda.codeset.nci.reverse}
      cda.value.add{target=SA, element='doseQuantity', datatype='PQ', value='50', unit='mcg'}
      
      FillConsumable4(SA)
      
      local ER = addElement(SA, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Immunization Refusal"]}   
      cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 Act No Immunization Reason"], 
         value=cda.codeset.actNoImmunization["Patient Objection"], lookup=cda.codeset.actNoImmunization.reverse} 
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')       
      setAttr(ER, 'typeCode', 'RSON')   
      setAttr(setAttr(setAttr(SA, 'classCode', 'SBADM'), 'moodCode', 'EVN'), 'negationInd', 'false')       
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Immunizations"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["History of immunizations"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'History of Immunizations')
   setInner(addElement(S, 'text'), [[
      <content ID="immunSect"/>
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Vaccine</th>
                  <th>Date</th>
                  <th>Status</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>
                      <content ID="immi1"/> Influenza, seasonal, IM </td>
                  <td>Nov 1999</td>
                  <td>Completed</td>
              </tr>
              <tr>
                  <td>
                      <content ID="immi2"/> Influenza, seasonal, IM </td>
                  <td>Dec 1998</td>
                  <td>Completed</td>
              </tr>
              <tr>
                  <td>
                      <content ID="immi3"/> Pneumococcal polysaccharide vaccine, IM </td>
                  <td>Dec 1998</td>
                  <td>Completed</td>
              </tr>
              <tr>
                  <td>
                      <content ID="immi4"/> Tetanus and diphtheria toxoids, IM </td>
                  <td>Refused</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   FillImmunization1(S)
   FillImmunization2(S)
   FillImmunization3(S)
   FillImmunization4(S)
   
   return T
end

local function FillInterventions(T)
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Interventions"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Interventions"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'INTERVENTIONS PROVIDED')
   setInner(addElement(S, 'text'), [[
      <list listType="ordered">
          <item>Therapeutic exercise intervention: knee extension, 3 sets, 10
          repetitions, 10-lb weight. </item>
          <item>Therapeutic exercise intervention: arm curl, 3 sets, 10
          repetitions, 15-lb weight </item>
      </list>
      ]])
   
   return T
end

local function FillMedicalEquipment(T)
   local function FillSupply1(S)
      local E = addElement(S, 'entry')
      local SUP = addElement(E, 'supply')
      cda.id.template.add{target=SUP, 
         id_type=cda.codeset.templates["Non-medicinal Supply Activity"]}
      cda.id.add{target=SUP, id_type='2413773c-2372-4299-bbe6-5b0f60664446'}
      cda.code.simple.add{target=SUP, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SUP, element='effectiveTime'}
      cda.time.add{target=ET, element='high', time='19991104000000+0500'}
      cda.value.add{target=SUP, element='quantity', datatype='INT', value='2'}
      local P = addElement(SUP, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Product Instance"]}
      cda.id.add{target=PR, 
         id_type='2413773c-2372-4299-bbe6-5b0f60664489'}
      local PD = addElement(PR, 'playingDevice')
      cda.code.add{target=PD, element='code', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Automatic implantable cardioverter/defibrillator"], lookup=cda.codeset.snomedCT.reverse}
      local SE = addElement(PR, 'scopingEntity')
      cda.id.add{target=SE, 
         id_type='eb936010-7b17-11db-9fe1-0800200c9b65'}
      setAttr(PR, 'classCode', 'MANU')
      setAttr(P, 'typeCode', 'PRD')   
      setAttr(setAttr(SUP, 'classCode', 'SPLY'), 'moodCode', 'EVN')        
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end

   local function FillSupply2(S)
      local E = addElement(S, 'entry')
      local SUP = addElement(E, 'supply')
      cda.id.template.add{target=SUP, 
         id_type=cda.codeset.templates["Non-medicinal Supply Activity"]}
      cda.id.add{target=SUP, id_type='230b0ab7-206d-42d8-a947-ab4f63aad795'}
      cda.code.simple.add{target=SUP, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SUP, element='effectiveTime'}
      cda.time.add{target=ET, element='high', time='19980101000000+0500'}
      cda.value.add{target=SUP, element='quantity', datatype='INT', value='1'}
      local P = addElement(SUP, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Product Instance"]}
      cda.id.add{target=PR, 
         id_type='eb936010-7b17-11db-9fe1-0800200c9a68'}
      local PD = addElement(PR, 'playingDevice')
      cda.code.add{target=PD, element='code', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Total hip replacement prosthesis"], lookup=cda.codeset.snomedCT.reverse}
      local SE = addElement(PR, 'scopingEntity')
      cda.id.add{target=SE, 
         id_type='0abea950-5b40-4b7e-b8d9-2a5ea3ac5500'}
      setText(addElement(SE, 'desc'), 'Good Health Prostheses Company')
      setAttr(PR, 'classCode', 'MANU')
      setAttr(P, 'typeCode', 'PRD')   
      setAttr(setAttr(SUP, 'classCode', 'SPLY'), 'moodCode', 'EVN')        
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end

   local function FillSupply3(S)
      local E = addElement(S, 'entry')
      local SUP = addElement(E, 'supply')
      cda.id.template.add{target=SUP, 
         id_type=cda.codeset.templates["Non-medicinal Supply Activity"]}
      cda.id.add{target=SUP, id_type='c4ffe98e-3cd3-4c54-b5bd-08ecb80379e0'}
      cda.code.simple.add{target=SUP, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SUP, element='effectiveTime'}
      cda.time.add{target=ET, element='high', time='19990101000000+0500'}
      cda.value.add{target=SUP, element='quantity', datatype='INT', value='1'}
      local P = addElement(SUP, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Product Instance"]}
      cda.id.add{target=PR, 
         id_type='eb936010-7b17-11db-9fe1-0800200c9a68'}     
      local PD = addElement(PR, 'playingDevice')
      cda.code.add{target=PD, element='code', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Wheelchair"], lookup=cda.codeset.snomedCT.reverse}
      local SE = addElement(PR, 'scopingEntity')
      cda.id.add{target=SE, 
         id_type='eb936010-7b17-11db-9fe1-0800200c9b67'}
      setText(addElement(SE, 'desc'), 'Good Health Durable Medical Equipment')
      setAttr(PR, 'classCode', 'MANU')
      setAttr(P, 'typeCode', 'PRD')   
      setAttr(setAttr(SUP, 'classCode', 'SPLY'), 'moodCode', 'EVN')        
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Medical Equipment"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["History of medical equipment"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'MEDICAL EQUIPMENT')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Supply/Device</th>
                  <th>Date Supplied</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>Automatic implantable cardioverter/defibrillator</td>
                  <td>Nov 1999</td>
              </tr>
              <tr>
                  <td>Total hip replacement prosthesis</td>
                  <td>1998</td>
              </tr>
              <tr>
                  <td>Wheelchair</td>
                  <td>1999</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   FillSupply1(S)
   FillSupply2(S)
   FillSupply3(S)
   
   return T
end

local function FillMedications(T)
   local function FillConsumable(Root)
      local MAN = addElement(addElement(Root, 'consumable'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Medication Information"]} 
      cda.id.add{target=MAN, 
         id_type='2a620155-9d11-439e-92b3-5d9815ff4ee8'}
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil HFA"], lookup=cda.codeset.rxnorm.reverse}
      cda.code.originalText.add{target=MC, reference='#MedSec_1'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil 0.09 MG/ACTUAT inhalant solution"], lookup=cda.codeset.rxnorm.reverse}   
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Medication Factory Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.null.set(cda.id.add{target=AE}, cda.null.flavor.NoInformation)
      cda.null.set(cda.demographic.address.add{target=AE}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, 
         id_type='2.16.840.1.113883.19.5.9999.139'}
      cda.demographic.name.simple.add{target=RO, name='Community Health and Hospitals'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      local P = addElement(Root, 'participant')
      local PR = addElement(P, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Medication Drug Vehicle"]}
      cda.code.add{target=PR, element='code', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Medication drug vehicle"], lookup=cda.codeset.snomedCT.reverse}
      local PE = setAttr(addElement(PR, 'playingEntity'), 'classCode', 'MMAT')
      cda.code.add{target=PE, element='code', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Aerosol"], lookup=cda.codeset.rxnorm.reverse}   
      cda.demographic.name.simple.add{target=PE, name='Aerosol'}   
      setAttr(P, 'typeCode', 'CSM')
      setAttr(PR, 'classCode', 'MANU')
      
      return MAN
   end
   
   local function FillSymptom(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Medication Indication"]} 
      cda.id.add{target=OB, value='45665', 
         id_type='db734647-fc99-424c-a864-7e3cda82e703'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Finding"], lookup=cda.codeset.snomedCT.reverse} 
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20070103000000+0500'}
      cda.null.set(cda.time.add{target=ET, element='high'}, cda.null.flavor.Unknown)
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Pneumonia"], lookup=cda.codeset.snomedCT.reverse}  
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')       
      setAttr(ER, 'typeCode', 'RSON')   
      
      return ER
   end
   
   local function FillProduct1(Root)
      local MAN = addElement(addElement(Root, 'product'), 'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Medication Information"]}
      cda.id.add{target=MAN, id_type='2a620155-9d11-439e-92b3-5d9815ff4ee8'}          
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil 0.09 MG/ACTUAT inhalant solution"], lookup=cda.codeset.rxnorm.reverse} 
      cda.code.originalText.add{target=MC, reference='#MedSec_1'} 
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil 0.09 MG/ACTUAT inhalant solution"], lookup=cda.codeset.rxnorm.reverse} 
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Medication Factory Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981823', id_type='2.16.840.1.113883.19.5.9999.456'}                
      cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland',
         state='OR', zip='99123', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AUT = addElement(Root, 'author')
      cda.null.set(cda.time.add{target=AUT, element='time'}, cda.null.flavor.Unknown)
      local AA = addElement(AUT, 'assignedAuthor')
      cda.id.add{target=AA, id_type='2a620155-9d11-439e-92b3-5d9815fe4de8'}  
      cda.null.set(cda.demographic.address.add{target=AA}, cda.null.flavor.Unknown)
      local AP = addElement(AA, 'assignedPerson')
      cda.demographic.name.add{target=AP, element='name', given='Henry', family='Seven', prefix='Dr.'}
      local ER = addElement(Root, 'entryRelationship')
      local ACT = addElement(ER, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Instructions"]}   
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Medication instructions"], lookup=cda.codeset.snomedCT.reverse} 
      setInner(addElement(ACT, 'text'), '<reference value="#MedSec_1"/> label in spanish ')
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'INT')       
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true') 
      
      return MAN
   end
   
   local function FillOrder(Root)
      local ER = addElement(Root, 'entryRelationship')
      local SUP = addElement(ER, 'supply')
      cda.id.template.add{target=SUP, 
         id_type=cda.codeset.templates["Medication Supply Order"]} 
      cda.null.set(cda.id.add{target=SUP}, cda.null.flavor.NoInformation)      
      cda.code.simple.add{target=SUP, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SUP, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20070103000000+0500'}
      cda.value.add{target=SUP, element='repeatNumber', datatype='INT', value='1'}
      cda.value.add{target=SUP, element='quantity', datatype='INT', value='75'}
      
      FillProduct1(SUP)
     
      setAttr(setAttr(SUP, 'classCode', 'SPLY'), 'moodCode', 'INT')       
      setAttr(ER, 'typeCode', 'REFR')  
      
      return ER      
   end

   local function FillProduct2(Root)
      local MAN = addElement(addElement(Root, 'product'),'manufacturedProduct')
      cda.id.template.add{target=MAN, 
         id_type=cda.codeset.templates["Medication Information"]} 
      cda.id.add{target=MAN, id_type='2a620155-9d11-439e-92b3-5d9815ff4ee8'}      
      local MAT = addElement(MAN, 'manufacturedMaterial')
      local MC = cda.code.add{target=MAT, element='code', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil 0.09 MG/ACTUAT inhalant solution"], lookup=cda.codeset.rxnorm.reverse}
      cda.code.originalText.add{target=MC, reference='#MedSec_1'}       
      cda.code.add{target=MC, element='translation', system=cda.codeset.cat.RxNorm, 
         value=cda.codeset.rxnorm["Proventil 0.09 MG/ACTUAT inhalant solution"], lookup=cda.codeset.rxnorm.reverse} 
      local ORG = addElement(MAN, 'manufacturerOrganization')
      cda.demographic.name.simple.add{target=ORG, name='Medication Factory Inc.'}
      setAttr(MAN, 'classCode', 'MANU')
      local PER = addElement(Root, 'performer')
      cda.null.set(cda.time.add{target=PER, element='time'}, cda.null.flavor.Unknown)
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981823', id_type='2.16.840.1.113883.19.5.9999.45'}   
      cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland',
         state='OR', zip='99123', country='US'} 
      cda.null.set(cda.demographic.phone.add{target=AE}, cda.null.flavor.Unknown)   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, element='name', given='Henry', family='Seven', prefix = 'Dr.'}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1393'} 
      cda.demographic.name.simple.add{target=RO, name='Community Health and Hospitals'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown) 
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      
      return MAN
   end
   
   local function FillDispense(SA)
      local ER = addElement(SA, 'entryRelationship')
      local SUP = addElement(ER, 'supply')
      cda.id.template.add{target=SUP, 
         id_type=cda.codeset.templates["Medication Dispense"]}   
      cda.id.add{target=SUP, value='cb734647-fc99-424c-a864-7e3cda82e704', id_type='1.2.3.4.56789.1'}
      cda.code.simple.add{target=SUP, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SUP, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20070103000000+0500'}
      cda.value.add{target=SUP, element='repeatNumber', datatype='INT', value='1'}
      cda.value.add{target=SUP, element='quantity', datatype='INT', value='75'}
      
      FillProduct2(SUP)
      
      setAttr(setAttr(SUP, 'classCode', 'SPLY'), 'moodCode', 'EVN')       
      setAttr(ER, 'typeCode', 'REFR')  
      
      return ER      
   end

   local function FillDrug(S)
      local E = addElement(S, 'entry')
      local SA = addElement(E, 'substanceAdministration')
      cda.id.template.add{target=SA, 
         id_type=cda.codeset.templates["Medication Activity"]}
      cda.id.add{target=SA, 
         id_type='cdbd33f0-6cde-11db-9fe1-0800200c9a66'}
      setInner(addElement(SA, 'text'), '<reference value="#MedSec_1"/> Proventil HFA')
      cda.code.simple.add{target=SA, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=SA, element='effectiveTime'} 
      cda.time.add{target=ET, element='low', time='20070103000000+0500'}
      cda.time.add{target=ET, element='high', time='20120515000000+0500'}
      cda.timeperiod.add{target=SA, element='effectiveTime', period='6', unit='h'} 
      cda.code.add{target=SA, element='routeCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Respiratory (Inhalation)"], lookup=cda.codeset.nci.reverse}
      cda.value.add{target=SA, element='doseQuantity', datatype='PQ', value='1', unit='mg/actuat'}
      cda.value.add{target=SA, element='rateQuantity', datatype='PQ', value='90', unit='ml/min'}
      local Dose = cda.null.set(cda.dose.add{target=SA, element='maxDoseQuantity'}, cda.null.flavor.Unknown)
      cda.null.set(cda.value.add{target=Dose, element='numerator', datatype='PQ'}, cda.null.flavor.Unknown)
      cda.null.set(cda.value.add{target=Dose, element='denominator', datatype='PQ'}, cda.null.flavor.Unknown)
      cda.code.add{target=SA, element='administrationUnitCode', system=cda.codeset.cat['NCI - FDA RouteOfAdministration'], 
         value=cda.codeset.nci["Inhalant"], lookup=cda.codeset.nci.reverse}
      
      FillConsumable(SA)
      FillSymptom(SA)
      FillOrder(SA)
      FillDispense(SA)
      
      local PC = addElement(SA, 'precondition')
      cda.id.template.add{target=PC, id_type=cda.codeset.templates["Precondition for Substance Administration"]}
      local C = addElement(PC, 'criterion')
      cda.code.add{target=C, element='code', system=cda.codeset.cat["HL7 ActCode"], 
         value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
      cda.code.add{target=C, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Wheezing"], lookup=cda.codeset.snomedCT.reverse}
      setAttr(setAttr(SA, 'classCode', 'SBADM'), 'moodCode', 'EVN')      
      setAttr(E, 'typeCode', 'DRIV')
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Medications With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["History of medication use"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'MEDICATIONS')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Medication</th>
                  <th>Directions</th>
                  <th>Start Date</th>
                  <th>Status</th>
                  <th>Indications</th>
                  <th>Fill Instructions</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>
                      <content ID="MedSec_1">Proventil HFA
                      </content>
                  </td>
                  <td>0.09 MG/ACTUAT inhalant solution, 2 puffs QID PRN
                  wheezing</td>
                  <td>20070103</td>
                  <td>Active</td>
                  <td>Pneumonia (233604007 SNOMED CT)</td>
                  <td>Generic Substitition Allowed</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   FillDrug(S)
      
   return T
end

local function FillPayers(T)
   local function FillProvider(Root)
      local PER = addElement(Root, 'performer')
      cda.id.template.add{target=PER, 
         id_type=cda.codeset.templates["Payer Performer"]}
      cda.timerange.add{target=PER, element='time'}
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, id_type='2.16.840.1.113883.19'}
      cda.code.add{target=AE, element='code', system=cda.codeset.cat["HL7 Role Class Relationship"], 
         value=cda.codeset.roleClass["invoice payor"], lookup=cda.codeset.roleClass.reverse}
      cda.demographic.address.add{target=AE, street='23 Insurance Road', city='Blue Bell', state='MA', 
         country='US', zip='02368', use=cda.codeset.address.Work}
      cda.demographic.phone.add{target=AE, phone='(781)555-1515', use=cda.codeset.address.Work}
      local RO = addElement(AE, 'representedOrganization')
      cda.demographic.name.simple.add{target=RO, name='Good Health Insurance'}
      cda.demographic.phone.add{target=RO, phone='(781)555-1515', use=cda.codeset.address.Work}
      cda.demographic.address.add{target=RO, street='23 Insurance Road', city='Blue Bell', state='MA', 
         country='US', zip='02368', use=cda.codeset.address.Work}
      setAttr(PER, 'typeCode', 'PRF')
      
      return PER
   end
   
   local function FillGuarantor(Root)
      local PER = addElement(Root, 'performer')
      cda.id.template.add{target=PER, id_type=cda.codeset.templates["Payer Performer"]}
      cda.timerange.add{target=PER, element='time'}
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, id_type='329fcdf0-7ab3-11db-9fe1-0800200c9a66'}
      cda.code.add{target=AE, element='code', system=cda.codeset.cat["HL7 Role Class Relationship"], 
         value=cda.codeset.roleClass.guarantor, lookup=cda.codeset.roleClass.reverse}
      cda.demographic.address.add{target=AE, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', use=cda.codeset.address.Home}
      cda.demographic.phone.add{target=AE, phone='(781)555-1212', use=cda.codeset.address.Home}   
      local AP = addElement(AE, 'assignedPerson')
      cda.demographic.name.add{target=AP, given='Adam', family='Everyman', prefix='Mr.'}
      setAttr(PER, 'typeCode', 'PRF')
      
      return PER
   end

   local function FillCoveredParty(Root)
      local PAR = addElement(Root, 'participant')
      cda.id.template.add{target=PAR, 
         id_type=cda.codeset.templates["Covered Party Participant"]}
      cda.timerange.add{target=PAR, element='time'}
      local PR = addElement(PAR, 'participantRole')
      cda.id.add{target=PR, value='1138345', id_type='14d4a520-7aae-11db-9fe1-0800200c9a66'}
      setAttr(PAR, 'typeCode', 'COV')
      cda.code.add{target=PR, element='code', system=cda.codeset.cat["HL7 Role Class Relationship"], 
         value=cda.codeset.roleClass.self, lookup=cda.codeset.roleClass.reverse}
      cda.demographic.address.add{target=PR, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', use=cda.codeset.address.Home} 
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.add{target=PE, given='Frank', family='Everyman', prefix='Mr.'} 
      setAttr(PR, 'classCode', 'PAT')
      
      return PAR
   end
   
   local function FillPolicyHolder(Root)
      local PAR = addElement(Root, 'participant')
      cda.id.template.add{target=PAR, 
         id_type=cda.codeset.templates["Policy Holder"]}
      local PR = addElement(PAR, 'participantRole')
      cda.id.add{target=PR, value='1138345', id_type='2.16.840.1.113883.19'}
      setAttr(PAR, 'typeCode', 'HLD')
      cda.demographic.address.add{target=PR, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', use=cda.codeset.address.Home} 
      setAttr(PR, 'classCode', 'PAT')
      
      return PAR 
   end
   
   local function FillProcedure(Root)
      local ER = addElement(Root, 'entryRelationship')
      local PRO = addElement(ER, 'procedure')
      cda.code.add{target=PRO, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT.Colonoscopy, lookup=cda.codeset.snomedCT.reverse}
      setAttr(setAttr(PRO, 'classCode', 'PROC'), 'moodCode', 'PRMS')
      setAttr(ER, 'typeCode', 'SUBJ')
      
      return ER
   end
   
   local function FillAuthorization(Root)  
      local ER = addElement(Root, 'entryRelationship')
      local ACT = addElement(ER, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Authorization Activity"]}
      cda.id.add{target=ACT, id_type='f4dce790-8328-11db-9fe1-0800200c9a66'}
      cda.null.set(cda.code.add{target=ACT, element='code'}, cda.null.flavor.Unknown)  
      
      FillProcedure(ACT)

      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'REFR')
      
      return ER
   end
   
   local function FillPolicy(Root)
      local ER = addElement(Root, 'entryRelationship')
      local ACT = addElement(ER, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Policy Activity"]}
      cda.id.add{target=ACT, id_type='3e676a50-7aac-11db-9fe1-0800200c9a66'}      
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat["HL7 Role Class Relationship"], 
         value=cda.codeset.roleClass.self, lookup=cda.codeset.roleClass.reverse}      
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
      
      FillProvider(ACT)
      FillGuarantor(ACT) 
      FillCoveredParty(ACT)
      FillPolicyHolder(ACT)
      FillAuthorization(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'COMP')      
      
      return ER
   end
   
   local function FillPayer(S)
      local E = addElement(S, 'entry')
      local ACT = addElement(E, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Coverage Activity"]}
      cda.id.add{target=ACT, id_type='1fe2cdd0-7aad-11db-9fe1-0800200c9a66'}
      cda.code.add{target=ACT, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Payment sources"], lookup=cda.codeset.loinc.reverse}      
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}

      FillPolicy(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV')      
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, id_type=cda.codeset.templates.Payers}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc.Payer, lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'INSURANCE PROVIDERS')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Payer name</th>
                  <th>Policy type / Coverage type</th>
                  <th>Policy ID</th>
                  <th>Covered party ID</th>
                  <th>Policy Holder</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>Good Health Insurance</td>
                  <td>Extended healthcare / Family</td>
                  <td>Contract Number</td>
                  <td>1138345</td>
                  <td>Patient's Mother</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   FillPayer(S)
   
   return T
end

local function FillPlanOfCare(T)
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Plan of Care"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Treatment plan"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'PLAN OF CARE')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Planned Activity</th>
                  <th>Planned Date</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>Colonoscopy</td>
                  <td>20120512</td>
              </tr>
          </tbody>
      </table>
      ]])

   local E = addElement(S, 'entry')
   local OB = addElement(E, 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Plan of Care Activity Observation"]}
   cda.id.add{target=OB, id_type='9a6d1bac-17d3-4195-89a4-1121bc809b4a'}
   cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Colonoscopy"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['New']}
   local ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='center', time='20120512000000+0500'}
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'RQO')        
   setAttr(E, 'typeCode', 'DRIV')
     
   local ACT = addElement(addElement(S, 'entry'), 'act')
   cda.id.template.add{target=ACT, 
      id_type=cda.codeset.templates["Plan of Care Act"]}
   cda.id.add{target=ACT, id_type='9a6d1bac-17d3-4195-89a4-1121bc809a5c'}   
   cda.code.add{target=ACT, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Colonoscopy"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['New']}
   ET = cda.timerange.add{target=ACT, element='effectiveTime'}
   cda.time.add{target=ET, element='center', time='20120512000000+0500'}
   setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'RQO')        
   
   local ENC = addElement(addElement(S, 'entry'), 'encounter')
   cda.id.template.add{target=ENC, 
      id_type=cda.codeset.templates["Plan of Care Activity Encounter"]}
   cda.id.add{target=ENC, id_type='9a6d1bac-17d3-4195-89a4-1121bc809b4d'}   
   cda.code.add{target=ENC, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Colonoscopy"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=ENC, element='statusCode', value=cda.codeset.status['New']}
   ET = cda.timerange.add{target=ENC, element='effectiveTime'}
   cda.time.add{target=ET, element='center', time='20120512000000+0500'}
   setAttr(setAttr(ENC, 'classCode', 'ENC'), 'moodCode', 'INT')
   
   local PRO = addElement(addElement(S, 'entry'), 'procedure')
   cda.id.template.add{target=PRO, 
      id_type=cda.codeset.templates["Plan of Care Activity Procedure"]}
   cda.id.add{target=PRO, id_type='9a6d1bac-17d3-4195-89c4-1121bc809b5a'}   
   cda.code.add{target=PRO, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Colonoscopy"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=PRO, element='statusCode', value=cda.codeset.status['New']}
   ET = cda.timerange.add{target=PRO, element='effectiveTime'}
   cda.time.add{target=ET, element='center', time='20120512000000+0500'}
   setAttr(setAttr(PRO, 'classCode', 'PROC'), 'moodCode', 'RQO')
   
   return T
end

local function FillProblemList(T)
   local function FillStatus1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Status Observation"]}
      cda.id.add{target=OB, id_type='ab1791b0-5c71-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.Status, lookup=cda.codeset.loinc.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#STAT1"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20080103000000+0500'}
      cda.time.add{target=ET, element='high', time='20090227130000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT.Resolved, lookup=cda.codeset.snomedCT.reverse}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'REFR') 
      
      return ER
   end
   
   local function FillAge1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Age Observation"]}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Age at onset"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.value.add{target=OB, element='value', datatype='PQ', value='57', unit='a'}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true') 
      
      return ER      
   end
   
   local function FillHealthStatus1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Health Status Observation"]}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Health Status"], lookup=cda.codeset.loinc.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#problems"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Alive and well"], lookup=cda.codeset.snomedCT.reverse}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'REFR') 
      
      return ER      
   end
   
   local function FillProblem1(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Observation"]}
      cda.id.add{target=OB, id_type='ab1791b0-5c71-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT.Complaint, lookup=cda.codeset.snomedCT.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#problem1"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20080103000000+0500'}
      cda.time.add{target=ET, element='high', time='20080103000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT.Pneumonia, lookup=cda.codeset.snomedCT.reverse}    
      
      FillStatus1(OB)
      FillAge1(OB)
      FillHealthStatus1(OB)
 
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'SUBJ')  
      
      return ER
   end
   
   local function FillStatus2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Status Observation"]}
      cda.id.add{target=OB, id_type='ab1791b0-5c71-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.Status, lookup=cda.codeset.loinc.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#STAT2"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20080103000000+0500'}
      cda.time.add{target=ET, element='high', time='20090227130000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT.Active, lookup=cda.codeset.snomedCT.reverse}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'REFR') 
      
      return ER
   end
   
   local function FillAge2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Age Observation"]}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Age at onset"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.value.add{target=OB, element='value', datatype='PQ', value='57', unit='a'}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(setAttr(ER, 'typeCode', 'SUBJ'), 'inversionInd', 'true') 
      
      return ER      
   end
   
   local function FillHealthStatus2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Health Status Observation"]}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Health Status"], lookup=cda.codeset.loinc.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#problems"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT["Alive and well"], lookup=cda.codeset.snomedCT.reverse}    
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'REFR') 
      
      return ER      
   end
   
   local function FillProblem2(Root)
      local ER = addElement(Root, 'entryRelationship')
      local OB = addElement(ER, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Problem Observation"]}
      cda.id.add{target=OB, id_type='ab1791b0-5c71-11db-b0de-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT.Complaint, lookup=cda.codeset.snomedCT.reverse}
      setInner(addElement(OB, 'text'), '<reference value="#problem2"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      local ET = cda.timerange.add{target=OB, element='effectiveTime'}
      cda.time.add{target=ET, element='low', time='20080103000000+0500'}
      cda.time.add{target=ET, element='high', time='20080103000000+0500'}
      cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
         value=cda.codeset.snomedCT.Asthma, lookup=cda.codeset.snomedCT.reverse}    
      
      FillStatus2(OB)
      FillAge2(OB)
      FillHealthStatus2(OB)
 
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      setAttr(ER, 'typeCode', 'SUBJ')  
      
      return ER
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Problems With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc["Problem list"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'PROBLEMS')
   setInner(addElement(S, 'text'), [[
      <content ID="problems"/>
      <list listType="ordered">
          <item>
              <content ID="problem1">Pneumonia </content>
              <content ID="stat1">Status: Resolved</content>
          </item>
          <item>
              <content ID="problem2">Asthma</content>
              <content ID="stat2">Status: Active</content>
          </item>
      </list>
      ]])
   
   local E = addElement(S, 'entry')
   local ACT = addElement(E, 'act')
   cda.id.template.add{target=ACT, 
      id_type=cda.codeset.templates["Problem Concern Act"]}
   cda.id.add{target=ACT, id_type='ec8a6ff8-ed4b-4f7e-82c3-e98e58b45de7'}      
   cda.code.add{target=ACT, element='code', system=cda.codeset.cat["HL7 Act Class"], 
      value=cda.codeset.actClass.concern, lookup=cda.codeset.actClass.reverse}      
   cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
   local ET = cda.timerange.add{target=ACT, element='effectiveTime'}   
   cda.time.add{target=ET, element='low', time='20080103000000+0500'}
   cda.time.add{target=ET, element='high', time='20080103000000+0500'}
   
   FillProblem1(ACT)
   
   setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
   setAttr(E, 'typeCode', 'DRIV')
  
   E = addElement(S, 'entry')
   ACT = addElement(E, 'act')
   cda.id.template.add{target=ACT, 
      id_type=cda.codeset.templates["Problem Concern Act"]}
   cda.id.add{target=ACT, id_type='ec8a6ff8-ed4b-4f7e-82c3-e98e58b45de7'}      
   cda.code.add{target=ACT, element='code', system=cda.codeset.cat["HL7 Act Class"], 
      value=cda.codeset.actClass.concern, lookup=cda.codeset.actClass.reverse}      
   cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
   ET = cda.timerange.add{target=ACT, element='effectiveTime'}   
   cda.time.add{target=ET, element='low', time='20080103000000+0500'}
   cda.time.add{target=ET, element='high', time='20080103000000+0500'}
   
   FillProblem2(ACT)
   
   setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')
   setAttr(E, 'typeCode', 'DRIV')
   
   return T  
end


local function FillProcedures(T)
   local function FillSpecimen(Root)
      local SPC = addElement(Root, 'specimen')
      local SPEC = addElement(SPC, 'specimenRole')
      cda.id.add{target=SPEC, id_type='c2ee9ee9-ae31-4628-a919-fec1cbb58683'}
      local SE = addElement(SPEC, 'specimenPlayingEntity')
      cda.code.add{target=SE, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colonic polyp sample"], lookup=cda.codeset.snomedCT.reverse}
      setAttr(SPEC, 'classCode', 'SPEC')
      setAttr(SPC, 'typeCode', 'SPC')  
      
      return SPC
   end
   
   local function FillPerformer1(Root)
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='2981823', id_type='2.16.840.1.113883.19.5.9999.456'}
      cda.demographic.address.add{target=AE, street='1001 Village Avenue', city='Portland', state='OR', 
         country='US', zip='99123'}
      cda.demographic.phone.add{target=AE, phone='555-555-5000', use=cda.codeset.address.Work}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5.9999.1393'}
      cda.demographic.name.simple.add{target=RO, name='Community Health and Hospitals'}
      cda.demographic.phone.add{target=RO, phone='555-555-5000', use=cda.codeset.address.Work}      
      cda.demographic.address.add{target=RO, street='1001 Village Avenue', city='Portland', state='OR', 
         country='US', zip='99123'}
      local PAR = addElement(Root, 'participant')
      local PR = addElement(PAR, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Product Instance"]}
      cda.id.add{target=PR, id_type='742aee30-21c5-11e1-bfc2-0800200c9a66'}
      local PD = addElement(PR, 'playingDevice')
      cda.code.add{target=PD, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colonoscope"], lookup=cda.codeset.snomedCT.reverse}
      local SE = addElement(PR, 'scopingEntity')
      cda.id.add{target=SE, id_type='eb936010-7b17-11db-9fe1-0800200c9b65'}
      setAttr(PR, 'classCode', 'MANU')
      setAttr(PAR, 'typeCode', 'DEV')      
      
      return PER
   end
   
   local function FillProcedure(S)
      local E = addElement(S, 'entry')
      local PROC = addElement(E, 'procedure')
      cda.id.template.add{target=PROC, 
         id_type=cda.codeset.templates["Procedure Activity Procedure"]}
      cda.id.add{target=PROC, id_type='d68b7e32-7810-4f5b-9cc2-acd54b0fd85d'}
      local C = cda.code.add{target=PROC, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colonoscopy"], lookup=cda.codeset.snomedCT.reverse}    
      cda.code.originalText.add{target=C, reference='#Proc1'} 
      cda.code.simple.add{target=PROC, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=PROC, element='effectiveTime', time='20120512000000+0500'}
      cda.null.set(cda.code.add{target=PROC, element='methodCode'}, cda.null.flavor.Unknown)
      cda.code.add{target=PROC, element='targetSiteCode', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colon part"], lookup=cda.codeset.snomedCT.reverse}
      
      FillSpecimen(PROC)     
      FillPerformer1(PROC)
      
      setAttr(setAttr(PROC, 'classCode', 'PROC'), 'moodCode', 'EVN')
      setAttr(E, 'typeCode', 'DRIV') 
      
      return E
   end
   
   local function FillPerformer2(Root)
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='1234', id_type='2.16.840.1.113883.19.5'}
      cda.demographic.address.add{target=AE, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', zip='02368'}
      cda.demographic.phone.add{target=AE, phone='555-555-1234', use=cda.codeset.address.Work}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5'}
      cda.demographic.name.simple.add{target=RO, name='Community Health and Hospitals'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      local PAR = addElement(Root, 'participant')
      local PR = addElement(PAR, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Service Delivery Location"]}
      local C = cda.code.add{target=PR, element='code', system=cda.codeset.cat["Healthcare Service Location"], 
         value=cda.codeset.healthcareServiceLocation["Gastroenterology Clinic"], lookup=cda.codeset.healthcareServiceLocation.reverse} 
      cda.demographic.address.add{target=PR, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', zip='02368'}
      cda.null.set(cda.demographic.phone.add{target=PR}, cda.null.flavor.Unknown)   
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.simple.add{target=PE, name='Community Gastroenterology Clinic'}
      setAttr(PE, 'classCode', 'PLC')
      setAttr(PR, 'classCode', 'SDLOC')
      setAttr(PAR, 'typeCode', 'LOC') 
      
      return PER
   end
   
   local function FillObservation(S)
      local E = addElement(S, 'entry')
      local OB = addElement(E, 'observation')
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Procedure Activity Observation"]}
      cda.id.add{target=OB, value='123456789', id_type='2.16.840.1.113883.19'}
      local C = cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colonic polypectomy"], lookup=cda.codeset.snomedCT.reverse}
      cda.code.originalText.add{target=C, reference='#Proc1'} 
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Aborted']}
      cda.time.add{target=OB, element='effectiveTime', time='20110203000000+0500'}
      cda.code.add{target=OB, element='priorityCode', system=cda.codeset.cat["HL7 Act Priority"], 
         value=cda.codeset.actPriority["Callback results"], lookup=cda.codeset.actPriority.reverse}
      cda.null.set(cda.code.add{target=OB, element='value'}, cda.null.flavor.Unknown)
      cda.null.set(cda.code.add{target=OB, element='methodCode'}, cda.null.flavor.Unknown)
      cda.code.add{target=OB, element='targetSiteCode', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Abdomen and pelvis"], lookup=cda.codeset.snomedCT.reverse}   
      
      FillPerformer2(OB)
      
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')  
      
      return E
   end
   
   local function FillPerformer3(Root)
      local PER = addElement(Root, 'performer')
      local AE = addElement(PER, 'assignedEntity')
      cda.id.add{target=AE, value='1234', id_type='2.16.840.1.113883.19'}
      cda.demographic.address.add{target=AE, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', zip='02368'}
      cda.demographic.phone.add{target=AE, phone='555-555-1234', use=cda.codeset.address.Work}
      local RO = addElement(AE, 'representedOrganization')
      cda.id.add{target=RO, id_type='2.16.840.1.113883.19.5'}
      cda.demographic.name.simple.add{target=RO, name='Community Health and Hospitals'}
      cda.null.set(cda.demographic.phone.add{target=RO}, cda.null.flavor.Unknown)
      cda.null.set(cda.demographic.address.add{target=RO}, cda.null.flavor.Unknown)
      local PAR = addElement(Root, 'participant')
      local PR = addElement(PAR, 'participantRole')
      cda.id.template.add{target=PR, 
         id_type=cda.codeset.templates["Service Delivery Location"]}
      local C = cda.code.add{target=PR, element='code', system=cda.codeset.cat["Healthcare Service Location"], 
         value=cda.codeset.healthcareServiceLocation["Gastroenterology Clinic"], lookup=cda.codeset.healthcareServiceLocation.reverse} 
      cda.demographic.address.add{target=PR, street='17 Daws Rd.', city='Blue Bell', state='MA', 
         country='US', zip='02368'}
      cda.null.set(cda.demographic.phone.add{target=PR}, cda.null.flavor.Unknown)   
      local PE = addElement(PR, 'playingEntity')
      cda.demographic.name.simple.add{target=PE, name='Community Gastroenterology Clinic'}
      setAttr(PE, 'classCode', 'PLC')
      setAttr(PR, 'classCode', 'SDLOC')
      setAttr(PAR, 'typeCode', 'LOC')   
      
      return PER
   end

   local function FillAct(S)
      local E = addElement(S, 'entry')
      local ACT = addElement(E, 'act')
      cda.id.template.add{target=ACT, 
         id_type=cda.codeset.templates["Procedure Activity Act"]}
      cda.id.add{target=ACT, value='1234567', id_type='1.2.3.4.5.6.7.8'}
      local C = cda.code.add{target=ACT, element='code', system=cda.codeset.cat.SNOMED_CT, 
         value=cda.codeset.snomedCT["Colonic polypectomy"], lookup=cda.codeset.snomedCT.reverse} 
      cda.code.originalText.add{target=C, reference='#Proc1'} 
      cda.code.simple.add{target=ACT, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=ACT, element='effectiveTime', time='20110203000000+0500'}
      cda.code.add{target=ACT, element='priorityCode', system=cda.codeset.cat["HL7 Act Priority"], 
         value=cda.codeset.actPriority["Callback results"], lookup=cda.codeset.actPriority.reverse} 

      FillPerformer3(ACT)
      
      setAttr(setAttr(ACT, 'classCode', 'ACT'), 'moodCode', 'EVN')  
      
      return E
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Procedures With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc["History of procedures"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'PROCEDURES')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Procedure</th>
                  <th>Date</th>
              </tr>
           </thead>
           <tbody>
               <tr>
                    <td>
                        <content ID="Proc1">Colonic polypectomy</content>
                    </td>
                    <td>1998</td>
               </tr>
          </tbody>
      </table>
      ]])

   FillProcedure(S)
   FillObservation(S)
   FillAct(S)
  
   return T
end

local function FillResults(T)
   local function FillResult1(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Result Observation"]} 
      cda.id.add{target=OB, id_type='107c2dc0-67a5-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.HGB, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#result1"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000323143000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='13.2', unit='g/dl'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      local OBR = addElement(addElement(OB, 'referenceRange'), 'observationRange')
      setText(addElement(OBR, 'text'), 'M 13-18 g/dl; F 12-16 g/dl')
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end

   local function FillResult2(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Result Observation"]} 
      cda.id.add{target=OB, id_type='107c2dc0-67a5-11db-bd13-0800200c9a66'}      
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.WBC, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#result2"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000323143000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='6.7', unit='10+3/ul'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      local OBR = addElement(addElement(OB, 'referenceRange'), 'observationRange')
      local Unit = '10+3/ul'
      local VR = cda.valuerange.add{target=OBR, element='value', datatype='IVL_PQ'}
      cda.value.add{target=VR, element='low', datatype='PQ', value='4.3', unit=Unit}
      cda.value.add{target=VR, element='high', datatype='PQ', value='10.8', unit=Unit}      
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')  
      
      return OB
   end
   
   local function FillResult3(Root)
      local OB = addElement(addElement(Root, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Result Observation"]}
      cda.id.add{target=OB, id_type='107c2dc0-67a5-11db-bd13-0800200c9a66'}      
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.PLT, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#result3"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000323143000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='123', unit='10+3/ul'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Low, lookup=cda.codeset.interpretation.reverse}
      local OBR = addElement(addElement(OB, 'referenceRange'), 'observationRange')
      local Unit = '10+3/ul'
      local VR = cda.valuerange.add{target=OBR, element='value', datatype='IVL_PQ'}
      cda.value.add{target=VR, element='low', datatype='PQ', value='150', unit=Unit}
      cda.value.add{target=VR, element='high', datatype='PQ', value='350', unit=Unit} 
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')  
      
      return OB
   end
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Results With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc["Results"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'RESULTS')
   setInner(addElement(S, 'text'), [[
      <table>
          <tbody>
              <tr>
                  <td colspan="2">LABORATORY INFORMATION</td>
              </tr>
              <tr>
                  <td colspan="2">Chemistries and drug levels</td>
              </tr>
              <tr>
                  <td>
                      <content ID="result1">HGB (M 13-18 g/dl; F 12-16
                      g/dl)</content>
                  </td>
                  <td>13.2</td>
              </tr>
              <tr>
                  <td>
                      <content ID="result2">WBC (4.3-10.8 10+3/ul)</content>
                  </td>
                  <td>6.7</td>
              </tr>
              <tr>
                  <td>
                      <content ID="result3">PLT (135-145 meq/l)</content>
                  </td>
                  <td>123 (L)</td>
              </tr>
              <tr>
                  <td colspan="2">Liver Functions and Other Laboratory Values</td>
              </tr>
              <tr>
                  <td>ALT (SGPT)</td>
                  <td>31.0</td>
              </tr>
              <tr>
                  <td>AST (SGOT)</td>
                  <td>18.0</td>
              </tr>
              <tr>
                  <td>GGT</td>
                  <td>28.0 Alk</td>
              </tr>
              <tr>
                  <td>Phos</td>
                  <td>86.0</td>
              </tr>
              <tr>
                  <td>Total Bili</td>
                  <td>0.1</td>
              </tr>
              <tr>
                  <td>Albumin</td>
                  <td>3.2</td>
              </tr>
              <tr>
                  <td colspan="2">Blood Count</td>
              </tr>
              <tr>
                  <td>White Count</td>
                  <td>7.7</td>
              </tr>
              <tr>
                  <td>Platelets</td>
                  <td>187.0</td>
              </tr>
              <tr>
                  <td>Hematocrit</td>
                  <td>23.7</td>
              </tr>
              <tr>
                  <td>Hemoglobin</td>
                  <td>8.1</td>
              </tr>
              <tr>
                  <td colspan="2">ELECTROCARDIOGRAM (EKG) INFORMATION</td>
              </tr>
              <tr>
                  <td>EKG</td>
                  <td>Sinus rhythm without acute changes</td>
              </tr>
          </tbody>
      </table>
      ]])
   
   local E = addElement(S, 'entry')
   local ORG = addElement(E, 'organizer')
   cda.id.template.add{target=ORG, 
      id_type=cda.codeset.templates["Result Organizer"]} 
   cda.id.add{target=ORG, id_type='7d5a02b0-67a4-11db-bd13-0800200c9a66'}
   cda.code.add{target=ORG, element='code', system=cda.codeset.cat.SNOMED_CT, 
      value=cda.codeset.snomedCT["CBC WO Differential"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
   
   FillResult1(ORG)
   FillResult2(ORG)
   FillResult3(ORG)
   
   setAttr(setAttr(ORG, 'classCode', 'BATTERY'), 'moodCode', 'EVN') 
   setAttr(E, 'typeCode', 'DRIV')
   
   return T
end

local function FillSocialHistory(T)
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Social History"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC,
      value=cda.codeset.loinc["Social history"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'SOCIAL HISTORY')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th>Social History Element</th>
                  <th>Description</th>
                  <th>Effective Dates</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <td>
                      <content ID="soc1"/> smoking </td>
                  <td>1 pack per day</td>
                  <td>20050501 to 20090227130000+0500</td>
              </tr>
              <tr>
                  <td>
                      <content ID="soc2"/> smoking </td>
                  <td>None</td>
                  <td>20090227130000+0500 - today</td>
              </tr>
              <tr>
                  <td>
                      <content ID="soc3"/> Alcohol consumption </td>
                  <td>None</td>
                  <td>20050501 - </td>
              </tr>
          </tbody>
      </table>
      ]])

   local E = addElement(S, 'entry')
   local OB = addElement(E, 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Smoking Status Observation"]}
   cda.id.add{target=OB, value='123456789', id_type='2.16.840.1.113883.19'}
   cda.code.add{target=OB, element='code', system=cda.codeset.cat["HL7 ActCode"],
      value=cda.codeset.act.Assertion, lookup=cda.codeset.act.reverse}
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
   local ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20050501000000+0500'}
   cda.time.add{target=ET, element='high', time='20090227130000+0500'}
   local V = cda.code.add{target=OB, element='value', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Former smoker"], lookup=cda.codeset.snomedCT.reverse}   
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
   setAttr(E, 'typeCode', 'DRIV')

   E = addElement(S, 'entry')
   OB = addElement(E, 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Social History Observation"]}
   cda.id.add{target=OB, id_type='9b56c25d-9104-45ee-9fa4-e0f3afaa01c1'}   
   local C = cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Tobacco use and exposure"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.originalText.add{target=C, reference='#soc1'} 
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
   ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20050501000000+0500'}
   cda.time.add{target=ET, element='high', time='20090227130000+0500'}   
   cda.valuestring.add{target=OB, string='1 pack per day'}
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
   setAttr(E, 'typeCode', 'DRIV')
   
   E = addElement(S, 'entry')
   OB = addElement(E, 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Social History Observation"]}
   cda.id.add{target=OB, id_type='45efb604-7049-4a2e-ad33-d38556c9636c'}      
   C = cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Tobacco use and exposure"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.originalText.add{target=C, reference='#soc2'} 
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
   ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20090227130000+0500'}
   cda.valuestring.add{target=OB, string='None'}
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
   setAttr(E, 'typeCode', 'DRIV')

   E = addElement(S, 'entry')
   OB = addElement(E, 'observation')
   cda.id.template.add{target=OB, 
      id_type=cda.codeset.templates["Social History Observation"]}
   cda.id.add{target=OB, id_type='37f76c51-6411-4e1d-8a37-957fd49d2cef'}         
   C = cda.code.add{target=OB, element='code', system=cda.codeset.cat.SNOMED_CT,
      value=cda.codeset.snomedCT["Alcohol consumption"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.originalText.add{target=C, reference='#soc3'} 
   cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
   ET = cda.timerange.add{target=OB, element='effectiveTime'}
   cda.time.add{target=ET, element='low', time='20050501000000+0500'}
   cda.valuestring.add{target=OB, string='None'}
   setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')        
   setAttr(E, 'typeCode', 'DRIV')

   return T
end

local function FillVitalSigns(T)
   local function FillResult1(ORG)
      local OB = addElement(addElement(ORG, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}      
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.Height, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit1"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='19991114000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='177', unit='cm'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end   
   
   local function FillResult2(ORG)
      local OB = addElement(addElement(ORG, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Patient body weight - measured"], lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit3"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='19991114000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='86', unit='kg'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end   
   
   local function FillResult3(ORG)
      local OB = addElement(addElement(ORG, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Intravascular systolic"], lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit5"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='19991114000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='132', unit='mm[Hg]'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end 
   
   local function FillResult4(ORG)
      local OB = addElement(addElement(ORG, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc.Height, lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit2"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000407000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='177', unit='cm'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end      
   
   local function FillResult5(ORG)
      local OB = addElement(addElement(ORG, 'component'), 'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Patient body weight - measured"], lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit4"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000407000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='88', unit='kg'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end   
   
   local function FillResult6(ORG)
      local OB = addElement(addElement(ORG, 'component'),'observation') 
      cda.id.template.add{target=OB, 
         id_type=cda.codeset.templates["Vital Signs Observation"]} 
      cda.id.add{target=OB, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
      cda.code.add{target=OB, element='code', system=cda.codeset.cat.LOINC, 
         value=cda.codeset.loinc["Intravascular systolic"], lookup=cda.codeset.loinc.reverse} 
      setInner(addElement(OB, 'text'), '<reference value="#vit6"/>')
      cda.code.simple.add{target=OB, element='statusCode', value=cda.codeset.status['Completed']}
      cda.time.add{target=OB, element='effectiveTime', time='20000407000000+0500'}
      cda.value.add{target=OB, element='value', datatype='PQ', value='145', unit='mm[Hg]'} 
      cda.code.add{target=OB, element='interpretationCode', system=cda.codeset.cat.ObservationInterpretation, 
         value=cda.codeset.interpretation.Normal, lookup=cda.codeset.interpretation.reverse}
      setAttr(setAttr(OB, 'classCode', 'OBS'), 'moodCode', 'EVN')
      
      return OB
   end 
   
   local S = addElement(T, 'section')
   cda.id.template.add{target=S, 
      id_type=cda.codeset.templates["Vital Signs With Entries"]}
   cda.code.add{target=S, element='code', system=cda.codeset.cat.LOINC, 
      value=cda.codeset.loinc["Vital signs"], lookup=cda.codeset.loinc.reverse}
   setText(addElement(S, 'title'), 'VITAL SIGNS')
   setInner(addElement(S, 'text'), [[
      <table border="1" width="100%">
          <thead>
              <tr>
                  <th align="right">Date / Time: </th>
                  <th>Nov 14, 1999</th>
                  <th>April 7, 2000</th>
              </tr>
          </thead>
          <tbody>
              <tr>
                  <th align="left">Height</th>
                  <td>
                      <content ID="vit1">177 cm</content>
                  </td>
                  <td>
                      <content ID="vit2">177 cm</content>
                  </td>
              </tr>
              <tr>
                  <th align="left">Weight</th>
                  <td>
                      <content ID="vit3">86 kg</content>
                  </td>
                  <td>
                      <content ID="vit4">88 kg</content>
                  </td>
              </tr>
              <tr>
                  <th align="left">Blood Pressure</th>
                  <td>
                      <content ID="vit5">132/86 mmHg</content>
                  </td>
                  <td>
                      <content ID="vit6">145/88 mmHg</content>
                  </td>
              </tr>
          </tbody>
      </table>
      ]])
      
   local E = addElement(S, 'entry')
   local ORG = addElement(E, 'organizer')
   cda.id.template.add{target=ORG, 
      id_type=cda.codeset.templates["Vital Signs Organizer"]} 
   cda.id.add{target=ORG, id_type='c6f88320-67ad-11db-bd13-0800200c9a66'}
   cda.code.add{target=ORG, element='code', system=cda.codeset.cat.SNOMED_CT, 
      value=cda.codeset.snomedCT["Vital signs"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
   cda.time.add{target=ORG, element='effectiveTime', time='19991114000000+0500'}
   
   FillResult1(ORG)
   FillResult2(ORG)
   FillResult3(ORG)

   setAttr(setAttr(ORG, 'classCode', 'CLUSTER'), 'moodCode', 'EVN') 
   setAttr(E, 'typeCode', 'DRIV')
   
   E = addElement(S, 'entry')
   ORG = addElement(E, 'organizer')
   cda.id.template.add{target=ORG, 
      id_type=cda.codeset.templates["Vital Signs Organizer"]} 
   cda.code.add{target=ORG, element='code', system=cda.codeset.cat.SNOMED_CT, 
      value=cda.codeset.snomedCT["Vital signs"], lookup=cda.codeset.snomedCT.reverse}
   cda.code.simple.add{target=ORG, element='statusCode', value=cda.codeset.status['Completed']}
   cda.time.add{target=ORG, element='effectiveTime', time='20000407000000+0500'}
   
   FillResult4(ORG)
   FillResult5(ORG)
   FillResult6(ORG)

   setAttr(setAttr(ORG, 'classCode', 'CLUSTER'), 'moodCode', 'EVN') 
   setAttr(E, 'typeCode', 'DRIV')   
   
   return T
end

function main(Data)   
   local Doc = cda.new()
   local CD = Doc.ClinicalDocument
   
   -- CDA Header 
   FillHeader(CD)
   local RT = addElement(CD, 'recordTarget')
   FillPatient(RT)
   local A = addElement(CD, 'author')
   FillAuthor(A)
   local DE = addElement(CD, 'dataEnterer')
   FillDataEnterer(DE) 
   local I = addElement(CD, 'informant')
   FillInformant1(I)
   I = addElement(CD, 'informant')
   FillInformant2(I)
   local C = addElement(CD, 'custodian')
   FillCustodian(C)
   local IR = addElement(CD, 'informationRecipient')
   FillInformationRecipient(IR)
   local LA = addElement(CD, 'legalAuthenticator')
   FillLegalAuthenticator(LA)
   local AU = addElement(CD, 'authenticator')
   FillAuthenticator(AU)
   local D = addElement(CD, 'documentationOf')
   FillServiceEvent(D)
   
   -- CDA Body
   local Body = addElement(CD, 'component')
   local SB = addElement(Body, 'structuredBody')
   local COM = addElement(SB, 'component')
   FillAdvancedDirectives(COM)
   COM = addElement(SB, 'component')   
   FillAllergies(COM)
   COM = addElement(SB, 'component')
   FillAssessment(COM)
   COM = addElement(SB, 'component')
   FillEncounters(COM)
   COM = addElement(SB, 'component')   
   FillFamilyHistory(COM)
   COM = addElement(SB, 'component')   
   FillFunctionalStatus(COM)
   COM = addElement(SB, 'component')   
   FillImmunizations(COM)   
   COM = addElement(SB, 'component') 
   FillInterventions(COM)   
   COM = addElement(SB, 'component')
   FillMedicalEquipment(COM)
   COM = addElement(SB, 'component')
   FillMedications(COM)
   COM = addElement(SB, 'component')
   FillPayers(COM)
   COM = addElement(SB, 'component')
   FillPlanOfCare(COM)
   COM = addElement(SB, 'component')
   FillProblemList(COM)   
   COM = addElement(SB, 'component')
   FillProcedures(COM)
   COM = addElement(SB, 'component')
   FillResults(COM)
   COM = addElement(SB, 'component')
   FillSocialHistory(COM)
   COM = addElement(SB, 'component')
   FillVitalSigns(COM)
   net.http.respond{body=tostring(Doc)}

   -- TEST CODE: write CDA to file (in Iguana install dir)
   if iguana.isTest() then
      -- unformatted xml
      local f = io.open('cda_full_xml.xml','w+')
      f:write(tostring(Doc))
      f:close()
      -- formatted with xsl stylesheet
      f = io.open('cda_web.xml','w+')
      f:write('<?xml-stylesheet type="text/xsl" href="WebViewLayout_CDA.xsl"?>\n')
      f:write(tostring(Doc))
      f:close()
   end
end